/******************************************************************************
*
*	   Copyright (C) 2004 Motorola, Inc.
*	   All Rights Reserved
*
* Filename:		$RCSfile: Echo.c,v $
* Author:		$Author: r22624 $
* Locker:		$Locker:  $
* State:		$State: Exp $
* Revision:		$Revision: 1.0 $
*
******************************************************************************
* Modification since 1.7.2006
******************************************************************************
*
* Freescale Semiconductor Inc.
* (c) Copyright 2004-2006 Freescale Semiconductor, Inc.
* (c) Copyright 2001-2004 Motorola, Inc.
* ALL RIGHTS RESERVED.
*
***************************************************************************//*!
*
* @file      Echo.c
*
* @author    B01119
* 
* @version   1.0.4.0
* 
* @date      May-20-2007
* 
* @brief     Echo.c - driver for MC33696
*
*******************************************************************************
*
* Functions:	...
*
* History:		28/7/04 - File created.
*				29/7/04 - Written initialisation function. Discovered something
*						  weird when STROBE is taken low: the SPI goes funny.
*				30/7/04 - Refined the initialisation and read registers
*						  functions. Factored code to read/write Echo registers
*						  into function Echo_Config. Investigated SPI error and
*						  written Enable function.
*				 2/8/04 - Investigated the strange behavious when STOBE is low.
*						  No answers yet; only more questions. It gets even
*						  worse when CONFB is low too. Added the disable
*						  function.
*				 3/8/04 - Errors caused by a delay necessary to allow Echo to
*						  power up out of Standyby mode: about 1.2ms!!!
*						  Written most of Rx state machine. Hopefully it works ;)
*				 4/8/04 - Worked on Tx state machine. Strange errors and
*						  behaviours appeared... May be due to RX_DATA_IT pin
*						  on my connector: have disconnected.
*				 5/8/04 - Errors caused by a combination of the RX_DATA_IT pin,
*						  the SPI interrupt being linked to the wrong place
*						  (oops!), and Echo outputting random stuff onto the
*						  SPI at max data rate in OOK modulation (OOKREF=1)
*						  with ID = 2A and header = 6.
*						  Have written and verified majority of Tx machine.
*						  The Wait state and SendMessage function are still
*						  outstanding. Remaining functions are mostly bit-
*						  -twiddling after this. And, of course, the RSSI ones.
*				 6/8/04 - Added SendMessage function and completed Wait state.
*				 9/8/04 - Added checksum calculation and sending to Tx side.
*						  Added double buffering onto the Tx side for SendMessage
*						  function. Written DriverStatus function. Written slow
*						  versions of the bit twiddling functions (excluding
*						  ChangeConfig and the RSSI functions).
*				10/8/04 - Removed large busy wait times when switching to Tx mode:
*						  additional states STARTUP and RXTX_SWITCH added to
*						  implement wait. Echo_Config broken into EnterConfig,
*						  Config and ExitConfig functions to accomodate delays.
*						  1.2ms busy waits now only present in Initialize,
*						  Enable and Disable functions. Further work necessary
*						  to determine if elimination of these busy delays is
*						  required.
*						  Changes today have reduced compiled code size by 76
*						  bytes!
*				11/8/04 - Completed and checked bit-twiddling functions with
*						  the exception of the SetFreq[NoFRM] family.
*						  Investigated communication problems with Echo v2.1.
*				17/8/04 - Finished off RSSI functions and ChangeConfig.
*						  Possible additions include time-out from Rx states,
*						  only copy necessary parts of buffer on Rx, copy
*						  buffer in asm for speed.
*				23/8/04 - Fixed some minor bugs discovered in testing e.g.
*						  calculation for RSSIC ticks, DOCONFIG Tx state jumped
*						  to SWITCHBACK (which assumes Tx mode) when DOCONFIG
*						  may be entered in Rx mode - Echo may be asleep in Rx
*						  mode and SWITCHBACK does not delay to allow Echo to
*						  wake up. Now DOCONFIG switches back to Rx mode on its
*						  own. Bug in Change_Config also fixed - cases were chars
*						  '1', '2' etc instead of 1,2 etc!
*				25/8/04 - Fixed bugs in Tx machine - extra '1' which appeared
*						  after ID and also the buffer full bit being sent as 1
*						  in the length byte.
*				03/9/04 - Added a timeout to the Rx machine. Also, added LVD bit
*						  to status and a function to clear the error and timeout
*						  flags.
*       21/11/06 - Fixed bug with STROBE oscillator - now started correctly.
*                  Fixed bug with last byte in receive mode, now driver can 
*             receive all messages with or without last dummy byte
*             Complete driver are modified for both 8-bit mcu HC08/HCS08
*       20/5/07 - complete tested and fixed some bugs. Driver fully support 
*             all new function of MC33696. Two function were removed 
*             void Echo_ReadDigitalRSSI(void);
*             void Echo_ReadAnalogueRSSI(void);
*
* Description:	The code for the Echo software device driver. Partially
*				based on Tango/Romeo drivers.
*
*
* Notes:
*
******************************************************************************/

#include "Echo.h"						/* Include Echo driver header file				*/

/* Prototypes of unexposed functions													*/
void WaitN(unsigned int n);
void EchoCalcChecksum(byte temp);
void EchoCopyTxBuf(void);
void Echo_EnterConfig(void);
void Echo_Config(byte command, byte *registerBuffer, byte count);
void Echo_ExitConfig(void);
void Echo_KickOffTimer(void);
void Echo_StartDelayedConfig(void);
void Echo_EndReceiveMsg(void);


#define SPI_TxBufFull  ((ECHO_SPIS & ECHO_SPIS_SPTEF) == 0)
#define SPI_RxBufEmpty ((ECHO_SPIS & ECHO_SPIS_SPRF) == 0)

/* Build a command byte - no checking of n, start or rw									*/
#define ECHO_REG_READ   0   /* Read registers */
#define ECHO_REG_WRITE  1   /* Write registers */
#define CMD_ONE		0					/*	n=0 --> 1 register							*/
#define CMD_TWO		1					/*	n=1 --> 2 register							*/
#define CMD_FOUR	2					/*	n=2 --> 4 register							*/
#define CMD_EIGHT	3					/*	n=3 --> 8 register							*/
#define BUILDCMD(n, start, rw)		((n<<6) | (start<<1) | rw)


//#pragma DATA_SEG SHORT MY_ZEROPAGE

tECHO_STATUS status;					/* Status flags									*/
tECHO_TXCONFIG tx_config;				/* Tx state machine configuration				*/
byte echoDriverNextState;		/* State to jump to on next interrupt			*/

#ifdef ECHO_PLUS_MODE
byte echoBankStatus = 0;    /* Save actual working bank */
#endif

//#pragma DATA_SEG DEFAULT


#if ECHO_BANK_A_EN == 1
  byte echoRegisters[13] = {		/* Local copy of Echos registers				*/
  	ECHO_CONFIG1_REG_VALUE,	ECHO_CONFIG2_REG_VALUE,	ECHO_CONFIG3_REG_VALUE,
  	ECHO_COMMAND_REG_VALUE,	ECHO_F1_REG_VALUE,		ECHO_F0_REG_VALUE,
  	ECHO_FT2_REG_VALUE,		ECHO_FT1_REG_VALUE,		ECHO_FT0_REG_VALUE,
  	ECHO_RXONOFF_REG_VALUE,	ECHO_ID_REG_VALUE,		ECHO_HEADER_REG_VALUE,
  	ECHO_RSSI_REG_VALUE
  	};
#endif	


byte IDCounter;				/* Number of ID's to repeat in continuous mode	*/

//#pragma DATA_SEG SHORT MY_ZEROPAGE
byte bitCounter;				/* Bits in current byte remaining to send		*/
#pragma DATA_SEG DEFAULT

byte byteCounter;				/* Number of bytes remaining to send			*/
byte data;						/* Byte being sent								*/
byte *echoTxDataPtr;			/* Pointer to retrieve data from Tx msg buffer	*/

/* Double buffering on the Tx side;	buffer format detailed in design doc.				*/
byte echoTransmitBuffer[ECHO_MAX_DATA_SIZE+4];
byte echoInternalTxBuffer[ECHO_MAX_DATA_SIZE+4];


byte echoChecksum;				/* Incremental checksum calculation				*/
byte echoDataLength;			/* Length stored between Rx interrupts			*/
byte *echoRxDataPtr;			/* Pointer to store data into Rx msg buffer		*/

/* Set up space for the receive buffers; buffer format detailed in design doc.			*/
byte echoRxBuffer[(ECHO_MAX_DATA_SIZE+2)*ECHO_RX_BUF_COUNT];
byte echoInternalRxBuffer[ECHO_MAX_DATA_SIZE+3];	// need extra byte for chksum

byte *echoNextMessage;			/* Pointer to the next full message buffer.		*/
										/* Only valid after a call to Echo_DriverStatus */
										/* which sets the MsgReady bit.					*/

byte *echoFullPtr;				/* Internal pointer to start of next full buf	*/
byte *echoEmptyPtr;			/* Internal pointer to start of next empty buf	*/




/* Configuration parameters held while waiting for Echo to turn on						*/
byte echoCommand;				/* The command to send Echo						*/
tECHO_WORD_PTR echoReg;				/* Buffer to read from or write to				*/
byte echoRegCounter;			/* The number of registers to read or write		*/

#ifdef ECHO_PLUS_MODE
  
  #define ECHO_ST_BANK_A 0x01
  #define ECHO_ST_BANK_B 0x02
  #define ECHO_ST_BANK_BOTH 0x04
  #define ECHO_ST_BANK_SWITCH 0x08
  #define ECHO_ST_BANK_WR_B 0x10
  #define ECHO_ST_BANK_WR_A 0x20
  
  
  
  #if ECHO_BANK_A_EN == 1 && ECHO_BANK_B_EN == 1
    const tSWITCH_BANK_INFO echoBankAConst = {
    ECHO_MODE_VALUE,                       
    ECHO_MODE_DSREF,                      
    ECHO_ID_VALUE,                         
    ECHO_ID_LENGTH,                        
    ECHO_HEADER_VALUE,                     
    ECHO_HEADER_LENGTH,                    
    ECHO_PREAMBLE_FSK00uS,                   
    ECHO_500uS_DELAY,                      
    ECHO_1200uS_DELAY,
    ECHO_2MS_DELAY,                     
    ECHO_MODH,                             
    ECHO_MODL,                             
    ECHO_COMH,                             
    ECHO_COML,                                                   
    ECHO_TIMER_MODULUS,                  
    };
  #else
  //const tSWITCH_BANK_INFO echoBankAConst;
  #define echoBankAConst *((char *) 0)  // TBD okentovat proc 0
  #endif
  
  const tSWITCH_BANK_INFO echoBankBConst = {
  ECHO_MODE_VALUE_B,                       
  ECHO_MODE_DSREF_B,                      
  ECHO_ID_VALUE_B,                         
  ECHO_ID_LENGTH_B,                        
  ECHO_HEADER_VALUE_B,                     
  ECHO_HEADER_LENGTH_B,                    
  ECHO_PREAMBLE_FSK00uS_B,                   
  ECHO_500uS_DELAY_B,                      
  ECHO_1200uS_DELAY_B,
  ECHO_2MS_DELAY_B,                     
  ECHO_MODH_B,                             
  ECHO_MODL_B,                             
  ECHO_COMH_B,                             
  ECHO_COML_B,                                                   
  ECHO_TIMER_MODULUS_B,                    
  };
  
//#pragma DATA_SEG SHORT MY_ZEROPAGE


  #if ECHO_BANK_A_EN == 1 && ECHO_BANK_B_EN == 1
    tSWITCH_BANK_INFO *p_echoBank = (tSWITCH_BANK_INFO*) &echoBankAConst;  
    //#define BANK_IS_A ((p_echoBank) == ((tSWITCH_BANK_INFO*) &echoBankAConst))
    #define BANK_IS_B ((p_echoBank) == ((tSWITCH_BANK_INFO*) &echoBankBConst))
  #else
    tSWITCH_BANK_INFO *p_echoBank = (tSWITCH_BANK_INFO*) &echoBankBConst;  
    #define BANK_IS_B ((p_echoBank) == ((tSWITCH_BANK_INFO*) &echoBankBConst))
  #endif  

#pragma DATA_SEG DEFAULT
  
  byte echoRegistersB[13] = {		/* Local copy of Echos registers				*/
	ECHO_CONFIG1_REG_VALUE_B,	ECHO_CONFIG2_REG_VALUE_B,	ECHO_CONFIG3_REG_VALUE_B,
	ECHO_COMMAND_REG_VALUE_B,	ECHO_F1_REG_VALUE,		ECHO_F0_REG_VALUE,
	ECHO_FT2_REG_VALUE,		ECHO_FT1_REG_VALUE,		ECHO_FT0_REG_VALUE,
	ECHO_RXONOFF_REG_VALUE_B,	ECHO_ID_REG_VALUE_B,		ECHO_HEADER_REG_VALUE_B,
	ECHO_RSSI_REG_VALUE
	};

  
#endif


#pragma MESSAGE INFORMATION DISABLE C4001  /* Condition is always true */
#pragma MESSAGE INFORMATION DISABLE C4002  /* Condition is always false */


/**---------------------------------------------------------------------------------------
 * Wait for n loop iterations. Adapted from GB60 LCD display driver in Metrowerks.
 */
#define ITERATIONS_6uS		(3*ECHO_MCU_BUS_SPEED/4000000)
#define ITERATIONS_10uS		(3*ECHO_MCU_BUS_SPEED/2400000)
#define ITERATIONS_1200uS	(3*ECHO_MCU_BUS_SPEED/20000)

void WaitN(unsigned int n) {
	asm {
		PSHX
		PSHH
		LDHX n
		loop:
		AIX #-1  ;2 cycles
		CPHX #0  ;3 cycles
		BNE loop ;3 cycles
		PULH
		PULX
	}
}



/**---------------------------------------------------------------------------------------
 * Helper function to calculate checksum.
 * Taken from the Romeo.c driver.
 */
void EchoCalcChecksum(byte temp) {
	asm {
		LDA echoChecksum
		ADD temp
		ADC #$00
		STA echoChecksum
	}
}


/**---------------------------------------------------------------------------------------
 * Copy the external Tx buffer into the internal Tx buffer.
 */
void EchoCopyTxBuf(void) {
	int i;
	for(i=0; i<sizeof(echoInternalTxBuffer); i++) {
		echoInternalTxBuffer[i] = echoTransmitBuffer[i];
	}

	echoInternalTxBuffer[3] |= ECHO_BUFFER_FULL;	/* Set buffer full flag				*/
	echoTransmitBuffer[3] &= ~ECHO_BUFFER_FULL;		/* Clear buffer full flag			*/
}

/**---------------------------------------------------------------------------------------
 * Put Echo into configuration mode. Enables the SPI, MCU as master and disables SPI
 * interrupts as required before calling Echo_Config.
 * Note: if Echo is already powered on, there should be a delay of approximately 5-6usec
 * (3*Tdigclk) before calling Echo_Config after this function to allow Echo to move into
 * config mode. If Echo may be in Standby mode, the delay required is 1.2ms!
 */
void Echo_EnterConfig(void) {
	ECHO_CONFB = 1;
	ECHO_SEB   = 1;

#ifdef __HCS08__                      /* Code for HCS08 mcu's           */
	ECHO_SPIC1 &= ~ECHO_SPIC1_SPIE;			/* Turn off SPI interrupts				*/
	ECHO_SPIC1 |= ECHO_SPIC1_MSTR;			/* Set MCU to master					*/
	ECHO_SPIC1 |= ECHO_SPIC1_SPE;				/* Enable SPI							*/ 
#else                                 /* Code for HC08 mcu's             */
       /* HC08 family MCUs code */   
  ECHO_SPIC1 = 0x2a;                  /* Enable SPI, turn off SPI interrupts, Master */
#endif

	ECHO_SEB   = 0;
	ECHO_CONFB = 0;

	WaitN(ITERATIONS_6uS);						  /* 3*Tdigclk (~5-6usec) delay			*/
}

/**---------------------------------------------------------------------------------------
 * Send the command byte and then read/write to and from the registerBuffer.
 * Assumes Echo is already in config mode and the SPI set up to talk to Echo (i.e.
 * Echo_EnterConfig has been called before this!).
 *
 * Expects SPI to be configured (baud rate, clock phase etc.) and CONFB and SEB to be
 * setup. No checking of count is performed to confirm this matches the command - make
 * sure it is correct! Writes do not destroy the original register buffer.
 */
void Echo_Config(byte command, byte *registerBuffer, byte count) {
	byte temp;							/* Temporary storage for reading SPI	*/
	byte writeFlag;					/* When writing, don't destroy the		*/
												/* register buffer by reading back		*/
												/* nothing from Echo and overwriting it	*/
	ECHO_CONFB = 1;
	#ifndef __HCS08__
	  ECHO_SPIC1 = 0;      /* Disable SPI */
	  ECHO_SPIC1 = 0x2A;   /* Enable SPI, turn off SPI interrupts, Master */
	#endif
	ECHO_CONFB = 0;

	/*--- The r/w bit is the LSB														*/
	writeFlag = command & 0x01;
	
	#ifdef ECHO_PLUS_MODE
	 
	  if(BANK_IS_B)        /* Acces to bank B ? */
	  {
	    temp = (command & 0b00111110) >> 1; // Copy actual register adress
	    temp += ECHO_REGB_OFFSET;  // Add offset to adress
	    command &= 0b11000001;     // Clear original adress
	    command |= (0b00111110 & (temp << 1)); // Set new adress to command */
	  }
	#endif

	temp = ECHO_SPIS;
	temp = ECHO_SPID;							/* Read data reg to clear receive flag	*/
  
	/*--- Send the command byte															*/
	ECHO_SPID = command;
	while(SPI_RxBufEmpty);     	/* Wait until byte gone					*/
	temp = ECHO_SPID;							/* Read data reg to clear receive flag	*/

	/*--- Now read/write the configuration bytes										*/	
	while(count>0) {
		ECHO_SPID = *registerBuffer;			/* Send a byte							*/
		while(SPI_RxBufEmpty);					/* Wait until byte gone					*/
		temp = ECHO_SPID;						/* Read a byte and clear receive flag	*/
		if (!writeFlag) *registerBuffer = temp; /* If a read command, copy to buf		*/
		registerBuffer++;
		count--;
	}
	
}

/**---------------------------------------------------------------------------------------
 * Exit configuration mode. Leaves SEB=1, CONFB=1, SPI enabled and MCU set as slave.
 */
void Echo_ExitConfig(void) {

	ECHO_CONFB = 1;								/* Leave config mode					*/
	ECHO_SEB = 1;								/* Disable Echo serial interface		*/

	ECHO_SPIC1 &= ~ECHO_SPIC1_MSTR;				/* Set MCU to slave						*/
}




/**---------------------------------------------------------------------------------------
 * Initialize Echo according to the config in Echo.h.
 * Echo is left in standby mode.
 */
void Echo_Initialize(void) {
	byte temp;
	/*--- Reset Status																	*/	
	status.Word = 0;
	
	/*--- Set up required pins															*/
	ECHO_CONFB = 1;
	ECHO_CONFB_DDR = 1;

	ECHO_SEB = 1;
	ECHO_SEB_DDR = 1;

  #ifdef ECHO_RSSIC
    ECHO_RSSIC = 0;
  	ECHO_RSSIC_DDR = 1;
  #endif

	/*--- Set up optional pins															*/
	#ifdef ECHO_ENABLEPA
		ECHO_ENABLEPA = 0;
		ECHO_ENABLEPA_DDR = 1;
	#endif

	#ifdef ECHO_ENABLELNA
		ECHO_ENABLELNA = 0;
		ECHO_ENABLELNA_DDR = 1;
	#endif


	#ifdef ECHO_STROBE
		ECHO_STROBE = 0;
		ECHO_STROBE_DDR = 1;
	#endif

	/*--- Init the SPI interface														*/
#ifdef __HCS08__	                /* Code for HCS08 mcu's                 */
	ECHO_SPIC1 = 0x04;							/* disabled, no int's, slave, clk setup	*/
	ECHO_SPIBR = ((ECHO_SPI_BAUD_RATE_PRESCALER << 4) | ECHO_SPI_BAUD_RATE_DIVISOR);
#else
  /* HC08 family MCUs code */
  ECHO_SPIC1 = 0x04;							/* disabled, no int's, slave, clk setup	*/
	ECHO_SPIS &= 0x0fc;
	ECHO_SPIS |= ECHO_SPI_BAUD_RATE_DIVISOR; 
  
#endif

	/*--- Write the registers															*/
	Echo_EnterConfig();
	WaitN(ITERATIONS_1200uS);
	#ifdef ECHO_PLUS_MODE
    // reset all registers
	  p_echoBank = (tSWITCH_BANK_INFO*) &echoBankAConst;
	#endif
	  temp = 0x08;  
	  Echo_Config(BUILDCMD(CMD_ONE,ECHO_CONFIG1_REG,1), &temp, 1);		/* reset registers	*/
	
	#if ECHO_BANK_A_EN == 1
	  #ifdef ECHO_PLUS_MODE     /* Set up Bank A */
	    p_echoBank = (tSWITCH_BANK_INFO*) &echoBankAConst;
	  #endif
	  Echo_Config(BUILDCMD(CMD_EIGHT,0,1), echoRegisters, 8);		/* first 8 registers	*/
	  Echo_Config(BUILDCMD(CMD_FOUR, 8,1), echoRegisters+8, 4);	/* next 4 registers		*/
	#endif
	#if ECHO_BANK_B_EN == 1	    /* Set up Bank B */
	  p_echoBank = (tSWITCH_BANK_INFO*) &echoBankBConst;
	  Echo_Config(BUILDCMD(CMD_EIGHT,0,1), echoRegistersB, 8);		/* first 8 registers	*/
	  Echo_Config(BUILDCMD(CMD_FOUR, 8,1), echoRegistersB + 8, 4);	/* next 4 registers		*/
	  #if ECHO_BANK_A_EN == 0   /* If bank A is Disabled, then set up SOE bit */
	    p_echoBank = (tSWITCH_BANK_INFO*) &echoBankAConst;
	    Echo_Config(BUILDCMD(CMD_ONE,ECHO_CONFIG2_REG,1), &echoRegistersB[ECHO_CONFIG2_REG], 1);		/* first 8 registers	*/
	    p_echoBank = (tSWITCH_BANK_INFO*) &echoBankBConst;
	  #endif  
	#endif
	Echo_ExitConfig();
	/* No need to send 13th byte - the RSSI is read only								*/

	ECHO_SPIC1 &= ~ECHO_SPIC1_SPE;				/* Disable SPI							*/

	/*--- Set up pointers																*/
	echoRxDataPtr = &echoInternalRxBuffer[0];	/* Point to internal Rx data buffer		*/

	echoFullPtr   = &echoRxBuffer[0];			/* Point full and empty to the Rx buf	*/
	echoEmptyPtr  = &echoRxBuffer[0];
}


/**---------------------------------------------------------------------------------------
 * Setup the timer, enable Echo and put into Rx mode.
 * A startup delay of 2ms is initiated.
 */
void Echo_Enable(void) {
	/*--- Indicate the driver status													*/
	status.Bits.Enabled = 1;
	status.Bits.EnableDelay = 1;
	status.Bits.Busy = 1;

	/*--- Turn on the LNA (for Rx)														*/
	#ifdef ECHO_ENABLELNA
		ECHO_ENABLELNA = 1;
	#endif

	/*--- Turn on the STROBE															*/
	#ifdef ECHO_STROBE
		ECHO_STROBE = 1;
	#endif

	/*--- Turn on Echo transceiver, and set mode to Rx									*/
	#if ECHO_BANK_A_EN == 1   /* For Bank A */
	  echoRegisters[ECHO_CONFIG2_REG] |= ECHO_BIT_TRXE;
	  echoRegisters[ECHO_COMMAND_REG] &= ~ECHO_BIT_MODE;
  #endif
  
  #if ECHO_BANK_B_EN == 1    /* For Bank B */
	  echoRegistersB[ECHO_CONFIG2_REG] |= ECHO_BIT_TRXE;
	  echoRegistersB[ECHO_COMMAND_REG] &= ~ECHO_BIT_MODE;
  #endif
  
  
	Echo_EnterConfig();
	WaitN(ITERATIONS_1200uS);
	#if ECHO_BANK_A_EN == 1
  	#ifdef ECHO_PLUS_MODE  /* Enable Bank A */
  	  p_echoBank = (tSWITCH_BANK_INFO*) &echoBankAConst;
  	#endif
  	Echo_Config(BUILDCMD(CMD_ONE, ECHO_COMMAND_REG, 1),
  	            &echoRegisters[ECHO_COMMAND_REG],
  	            1);
  	Echo_Config(BUILDCMD(CMD_ONE, ECHO_CONFIG2_REG, 1),
  	            &echoRegisters[ECHO_CONFIG2_REG],
  	            1);
	#endif
	#if ECHO_BANK_B_EN == 1  /* Enable Bank B */
  	p_echoBank = (tSWITCH_BANK_INFO*) &echoBankBConst;
  	Echo_Config(BUILDCMD(CMD_ONE, ECHO_COMMAND_REG, 1),
  	            &echoRegistersB[ECHO_COMMAND_REG],
  	            1);
  	Echo_Config(BUILDCMD(CMD_ONE, ECHO_CONFIG2_REG, 1),
  	            &echoRegistersB[ECHO_CONFIG2_REG],
  	            1);
  	#if ECHO_BANK_A_EN == 1
	    p_echoBank = (tSWITCH_BANK_INFO*) &echoBankAConst;
	  #endif            
	#endif
	
	Echo_ExitConfig();

	/*--- Disable SPI so timer channel pin can be used									*/
	ECHO_SPIC1 &= ~ECHO_SPIC1_SPE;
	
	/*--- Setup the timer and schedule 2ms delay										*/
	bitCounter = (byte) _ECHO_2MS_DELAY;				/* Counter = # of bit-times in 2ms		*/
	echoDriverNextState = ECHO_ENABLE_DELAY;

  #if ECHO_TIMER_DISABLE != 1
    ECHO_TPMxCxSC = ECHO_COMPARE_NO_OUTPUT;		/* Clear any old interrupt pending		*/
    ECHO_TPMxCNTL = 0;
  #endif

	ECHO_TPMxMODH = _ECHO_MODH;					/* Timer modulus = a bit-time			*/
	ECHO_TPMxMODL = _ECHO_MODL;
	
	ECHO_TPMxCxVH = _ECHO_MODH;					/* Compare = full bit-time				*/
	ECHO_TPMxCxVL = _ECHO_MODL;

	ECHO_TPMxCxSC &= ~ECHO_TPMxCxSC_CHnF;		/* Clear any old interrupt pending		*/
	ECHO_TPMxCxSC = ECHO_CLEAR_ON_COMPARE;		/* Set up interrupt on output compare	*/

#ifdef __HCS08__                      /* Code for HCS08 mcu's       */	
	ECHO_TPMxSC |= ECHO_TIMER_ON;				/* Start the timer						*/
#else
  /* HC08 family MCUs code */
  ECHO_TPMxSC &= ECHO_TIMER_ON;
#endif
  status.Bits.SwitchMode = 0;
 #ifdef ECHO_PLUS_MODE              /* Set up Bank switchnig status register */
   #if ECHO_BANK_A_EN == 1 && ECHO_BANK_B_EN == 1
    echoBankStatus |= ECHO_ST_BANK_BOTH;   /* Both banks */
    status.Bits.SwitchMode = 1;
   #else
    echoBankStatus |= ECHO_ST_BANK_B;      /* Bank B is set */
   #endif 
 #endif
	
	/*--- Enable interrupts																*/
	asm cli;
}


/**---------------------------------------------------------------------------------------
 * Put Echo into Standby/low power mode. Messages already received can be retrieved from
 * the driver buffer using Echo_DriverStatus as before but no new messages will be
 * received and nothing can be transmitted.
 *
 * !!Only Echo_Enable and Echo_DriverStatus are safe to be called when Echo is disabled!!
 */
void Echo_Disable(void) {
	/*--- Turn off timer channel and SPI interrupts										*/
#ifdef __HCS08__                            /* Code for HCS08 mcu's       */	
	ECHO_TPMxCxSC = ECHO_TIMER_PIN_OFF;
	ECHO_SPIC1 &= ~ECHO_SPIC1_SPIE;

	/*--- Turn off SPI module and timer													*/
	ECHO_SPIC1 &= ~ECHO_SPIC1_SPE;
	#if ECHO_TIMER_DISABLE
		ECHO_TPMxSC &= ECHO_TIMER_OFF;
	#endif

#else
   /* HC08 family MCUs code */ 
  ECHO_TPMxCxSC = ECHO_TIMER_PIN_OFF;
	ECHO_SPIC1 &= ~ECHO_SPIC1_SPIE;

	/*--- Turn off SPI module and timer													*/
	ECHO_SPIC1 &= ~ECHO_SPIC1_SPE;
	#if ECHO_TIMER_DISABLE
		ECHO_TPMxSC |= ECHO_TIMER_OFF;
	#endif
 
#endif

	/*--- Reset the driver status														*/
	status.Word = 0;

	/*--- Turn off the PA and LNA														*/
	#ifdef ECHO_ENABLELNA
		ECHO_ENABLELNA = 0;
	#endif

	#ifdef ECHO_ENABLEPA
		ECHO_ENABLEPA = 0;
	#endif

	/*--- Turn off RSSIE module and Echo transceiver									*/
	#if ECHO_BANK_A_EN == 1   /* Disable Bank A */
	  echoRegisters[ECHO_COMMAND_REG] &= ~ECHO_BIT_RSSIE;
	  echoRegisters[ECHO_CONFIG2_REG] &= ~ECHO_BIT_TRXE;
  #endif
  
  #if ECHO_BANK_B_EN == 1   /* Disable Bank B */
	  echoRegistersB[ECHO_COMMAND_REG] &= ~ECHO_BIT_RSSIE;
	  echoRegistersB[ECHO_CONFIG2_REG] &= ~ECHO_BIT_TRXE;
  #endif
  
	Echo_EnterConfig();
	WaitN(ITERATIONS_1200uS);
	#if ECHO_BANK_A_EN == 1
	  #ifdef ECHO_PLUS_MODE       /* Disable Bank A */
	    p_echoBank = (tSWITCH_BANK_INFO*) &echoBankAConst;
	  #endif    
	  Echo_Config(BUILDCMD(CMD_ONE, ECHO_COMMAND_REG, 1), &echoRegisters[ECHO_COMMAND_REG], 1);
	  Echo_Config(BUILDCMD(CMD_ONE, ECHO_CONFIG2_REG, 1), &echoRegisters[ECHO_CONFIG2_REG], 1);
	#endif
	#if ECHO_BANK_B_EN == 1        /* Disable Bank B */
	  p_echoBank = (tSWITCH_BANK_INFO*) &echoBankBConst;
	  Echo_Config(BUILDCMD(CMD_ONE, ECHO_COMMAND_REG, 1), &echoRegistersB[ECHO_COMMAND_REG], 1);
	  Echo_Config(BUILDCMD(CMD_ONE, ECHO_CONFIG2_REG, 1), &echoRegistersB[ECHO_CONFIG2_REG], 1);
	  #if ECHO_BANK_A_EN == 1
	    p_echoBank = (tSWITCH_BANK_INFO*) &echoBankAConst;
	  #endif    
	#endif
	Echo_ExitConfig();
	/* Echo_ExitConfig leaves SEB=1, digital interface disabled							*/

	/*--- Turn off the STROBE															*/
	/* After SPI stuff in case we're in Rx mode - STROBE=0 would turn Echo off before ^	*/
	#ifdef ECHO_STROBE
		ECHO_STROBE = 0;
	#endif
}

/**---------------------------------------------------------------------------------------
 * Set up timer registers and start it going. Require to setup echoDriverNextState
 * variable before starting timer interrupts! Also, SPI should be disabled while the timer
 * is generating a waveform.
 */
void Echo_KickOffTimer(void) {
	#if ECHO_TIMER_DISABLE != 1
    ECHO_TPMxCxSC = ECHO_COMPARE_NO_OUTPUT;		/* Clear any old interrupt pending		*/    
  #endif  
	
	/* Set up the timer																	*/
	ECHO_TPMxMODH = _ECHO_MODH;					/* Timer modulus = a bit-time			*/
	ECHO_TPMxMODL = _ECHO_MODL;
	
	ECHO_TPMxCxVH = _ECHO_COMH;					/* Compare = half bit-time				*/
	ECHO_TPMxCxVL = _ECHO_COML;

	ECHO_TPMxCxSC &= ~ECHO_TPMxCxSC_CHnF;		/* Clear any old interrupt pending		*/
	ECHO_TPMxCxSC = ECHO_COMPARE_NO_OUTPUT;		/* Set up interrupt on output compare	*/

  ECHO_TPMxCNTL = 0;
#ifdef __HCS08__                          /* Code for HCS08 mcu's            */
	ECHO_TPMxSC |= ECHO_TIMER_ON;				/* Start the timer						*/
#else
   /* HC08 family MCUs code */
  ECHO_TPMxSC &= ECHO_TIMER_ON;				/* Start the timer						*/ 
#endif
	
	/*--- Enable interrupts																*/
	asm cli;
}

/**---------------------------------------------------------------------------------------
 * Send a single preamble/ID sequence.
 * Switches to Tx mode and initiates the sending. While the Preamble/ID sequence is being
 * send, the Busy and Tx flags will be set. Note: does not return from Tx mode! Must be
 * used in combination with Echo_SendData to complete the message telegram and return
 * Echo to Rx mode.
 */
void Echo_SendPreambleID(void) {
	
	#if ECHO_BANK_A_EN == 1 && ECHO_BANK_B_EN == 1
    if(echoBankStatus & ECHO_ST_BANK_BOTH)
    {
      status.Bits.Error = 1;
      return;
    }
  #endif
  
	status.Bits.Busy = 1;
	if (status.Bits.Mode == 0) {
		/* If we're in Rx mode, need to switch to Tx									*/
		Echo_EnterConfig();
		bitCounter = _ECHO_1200uS_DELAY;
		echoDriverNextState = ECHO_STARTUP;
	}
	else {
		/* Jump straight to the preamble												*/
  	  if(_ECHO_MODE_VALUE == ECHO_OOK)
  	  {
  	    bitCounter = _ECHO_PREAMBLE_FSK00uS;
  			echoDriverNextState = ECHO_PREAMBLE_OOK;  
  	  }else
  	  {
  	    bitCounter = 4;
  			echoDriverNextState = ECHO_PREAMBLE_FSK;
  	  }
	}

	/* Configure the Tx state machine for ID only										*/
	tx_config.Bits.IDHeaderSelect = 0;
	tx_config.Bits.CompleteSend = 0;

	/* Buffer to pick up ID from														*/
	echoTxDataPtr = echoTransmitBuffer;

	Echo_KickOffTimer();
}


/**---------------------------------------------------------------------------------------
 * Send a preamble+Header+Data+EOM sequence.
 * Requires driver to be in Tx mode (by calling Send_PreambleID previously).
 */
void Echo_SendData(void) {
	/* Must be in Tx mode already (by calling Send_PreambleID)							*/
	if (status.Bits.Mode != 1) {
		status.Bits.Error = 1;
		return;
	}
  #if ECHO_BANK_A_EN == 1 && ECHO_BANK_B_EN == 1
    if(echoBankStatus & ECHO_ST_BANK_BOTH)
    {
      status.Bits.Error = 1;
      return;
    }
  #endif
	/* Status bit set to indicate transmitting											*/
	status.Bits.Busy = 1;
	status.Bits.Tx = 1;

	/* Configure the Tx state machine for P+Header+Data									*/
	tx_config.Bits.IDHeaderSelect = 1;

	/* Set up data, length, checksum etc												*/
	echoTransmitBuffer[3] |= ECHO_BUFFER_FULL;	/* Mark buf full while we're using it	*/

	echoTxDataPtr = echoTransmitBuffer;
	byteCounter = echoTransmitBuffer[3] & ~ECHO_BUFFER_FULL;
	echoChecksum = echoTxDataPtr[2];			/* Start with ID						*/

	/* Set up driver next state															*/
    if(_ECHO_MODE_VALUE == ECHO_OOK)   /* If driver works in switching, function must be run-time */
  	{	
  		echoDriverNextState = ECHO_PREAMBLE_OOK;
  		bitCounter = _ECHO_PREAMBLE_FSK00uS;
  	}else
  	{
  		echoDriverNextState = ECHO_PREAMBLE_FSK;
  		bitCounter = 4;
  	}

	Echo_KickOffTimer();
}


/**---------------------------------------------------------------------------------------
 * Send a complete telegram consisting of number of Preamble+ID sequences spaced by a
 * number of bit-times. The spaceing and repeat count are defined in the Tx buffer. This
 * sequence is followed by a Header, Data and EOM. Switches to Tx mode and initiates the
 * sending. While the sequence is being sent, the Busy and Tx flags will be set. The full
 * flag on the Tx buffer will remain set for the duration of its use by the driver. The
 * buffer may be copied internally, so even though the Tx buffer is empty, it might still
 * be sending!! Use the status flags to check.
 */
void Echo_SendMessage(void) {
  #if ECHO_BANK_A_EN == 1 && ECHO_BANK_B_EN == 1
    if(echoBankStatus & ECHO_ST_BANK_BOTH)
    {
      status.Bits.Error = 1;
      return;
    }
  #endif

	echoTransmitBuffer[3] |= ECHO_BUFFER_FULL;	/* Mark buf full while we're using it	*/

	/* The internal buffer is only full when transmitting a complete telegram. In that	*/
	/* case, the ISR will pick up the new message when it's finished, so just leave it.	*/
	if ((echoInternalTxBuffer[3] & ECHO_BUFFER_FULL) != 0) return;

	status.Bits.Busy = 1;

	/* Set up driver next state: Startup switches to Tx mode before preamble			*/
	Echo_EnterConfig();
	bitCounter = _ECHO_1200uS_DELAY;
	echoDriverNextState = ECHO_STARTUP;

	/* Configure the Tx state machine for P+ID repeat									*/
	tx_config.Bits.IDHeaderSelect = 0;
	tx_config.Bits.CompleteSend = 1;
	tx_config.Bits.IDRepeat = 0;

	/* Set up data, length, checksum etc												*/
	EchoCopyTxBuf();

	echoTxDataPtr = echoInternalTxBuffer;
	byteCounter = echoTxDataPtr[3] & ~ECHO_BUFFER_FULL;
	echoChecksum = echoTxDataPtr[2];			/* Start with ID						*/

	IDCounter = echoTxDataPtr[0];				/* Number of P+ID repeats				*/

	Echo_KickOffTimer();
}


/**---------------------------------------------------------------------------------------
 * Send a complete message consisting of Preamble followed my a continuous stream of ID's
 * then a Header, Data and EOM. Switches to Tx mode and initiates the sending. While the
 * sequence is being sent, the Busy and Tx flags will be set. Always sends at least one
 * ID; repeatCount is number of extra ID's to send.
 */
void Echo_SendIDRepeat(byte repeatCount) {
	#if ECHO_BANK_A_EN == 1 && ECHO_BANK_B_EN == 1
    if(echoBankStatus & ECHO_ST_BANK_BOTH)
    {
      status.Bits.Error = 1;
      return;
    }
  #endif
	status.Bits.Busy = 1;

	/* Set up driver next state: Startup switches to Tx mode before preamble			*/
	Echo_EnterConfig();
	bitCounter = _ECHO_1200uS_DELAY;
	echoDriverNextState = ECHO_STARTUP;

	/* Configure the Tx state machine for ID repeat										*/
	tx_config.Bits.IDHeaderSelect = 0;
	tx_config.Bits.CompleteSend = 1;
	tx_config.Bits.IDRepeat = 1;

	/* Set up data, length, checksum etc												*/
	echoTransmitBuffer[3] |= ECHO_BUFFER_FULL;	/* Mark buf full while we're using it	*/

	echoTxDataPtr = echoTransmitBuffer;
	byteCounter = echoTransmitBuffer[3] & ~ECHO_BUFFER_FULL;
	echoChecksum = echoTxDataPtr[2];			/* Start with ID						*/

	IDCounter = repeatCount;

	Echo_KickOffTimer();
}



/**---------------------------------------------------------------------------------------
 * Timer interrupt used for transmitting bits and other timing.
 * Essentially a big function implementing a state machine which transitions on
 * interrupts. See the Echo Device Driver Design document for a fuller description.
 */
interrupt void Echo_TxTimer_Interrupt(void) {
	byte temp;

	/*--- Read CHnF so that a later write will clear interrupt flag						*/
	temp = ECHO_TPMxCxSC;

	switch (echoDriverNextState) {

		/*- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
		/* 2ms delay when Echo starts up												*/
		case ECHO_ENABLE_DELAY:
			bitCounter--;

			if (bitCounter == 0) {
				status.Bits.EnableDelay = 0;
				status.Bits.Busy = 0;

				ECHO_TPMxCxSC &= ~ECHO_TPMxCxSC_CHnIE;	/* Turn off channel interrupts	*/
				ECHO_TPMxCxSC = ECHO_TIMER_PIN_OFF;		/* Turn off channel interrupts	*/

			#ifdef __HCS08__                          /* Code for HCS08 mcu's       */ 

        ECHO_SPIC1 |= ECHO_SPIC1_SPIE;			/* Enable SPI interrupts		*/
			  ECHO_SPIC1 |= ECHO_SPIC1_SPE;			/* Turn on the SPI				*/    
				#if ECHO_TIMER_DISABLE == 1
					ECHO_TPMxSC &= ECHO_TIMER_OFF;
				#endif


      #else
         /* HC08 family MCUs code */
			  ECHO_SPIC1 = 0x8a;			/* Enable receiver and SPI Interrupts		*/  
 				#if ECHO_TIMER_DISABLE == 1
					ECHO_TPMxSC |= ECHO_TIMER_OFF;
				#endif
 
      #endif

				ECHO_SEB = 0;							/* Enable Echo digital interface*/
														/* to receive msg's on the SPI	*/

				echoDriverNextState = ECHO_READY;
			}
			else {						/* Still more delay to come so go round again	*/
				ECHO_TPMxCxSC = ECHO_CLEAR_ON_COMPARE;
			}

			break;


		/*- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
		/* Startup: 1.2ms delay when Echo enters config mode to switch to Tx mode		*/
		/* Enter with bitCounter = # of bit-times in 1.2ms								*/
		/* Echo_EnterConfig should be called prior to entering this state				*/
		case ECHO_STARTUP:
			ECHO_TPMxCxSC = ECHO_COMPARE_NO_OUTPUT;    
			if (--bitCounter == 0) {
			  ECHO_CONFB = 1;
        WaitN(ITERATIONS_10uS); 
        ECHO_CONFB = 0;	
        echoDriverNextState = ECHO_RXTX_SWITCH;
			}
			break;


		/*- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
		/* RxTxSwitch: Switch to Tx mode before sending Preamble						*/
		case ECHO_RXTX_SWITCH:
			ECHO_TPMxCxSC = ECHO_COMPARE_NO_OUTPUT;

			/* Status bits set to Tx mode												*/
			status.Bits.Mode = 1;
			status.Bits.Rx   = 0;
			status.Bits.Tx   = 1;

			/* Turn on the PA and turn off the LNA										*/
			#ifdef ECHO_ENABLELNA
				ECHO_ENABLELNA = 0;
			#endif

			#ifdef ECHO_ENABLEPA
				ECHO_ENABLEPA = 1;
			#endif

			/* Switch to Tx mode														*/
			#ifdef ECHO_MODE        /* Simply mode of driver */
			  echoRegisters[ECHO_COMMAND_REG] |= ECHO_BIT_MODE;
			  Echo_Config(BUILDCMD(CMD_ONE, ECHO_COMMAND_REG, 1), &echoRegisters[ECHO_COMMAND_REG], 1);
			#else                  /* Switching mode of driver */
			  #if ECHO_BANK_A_EN == 1 /* Bank Switching mode */
  			  if(echoBankStatus & ECHO_ST_BANK_B)
  			  {
  			    echoRegistersB[ECHO_COMMAND_REG] |= ECHO_BIT_MODE;
  			    Echo_Config(BUILDCMD(CMD_ONE, ECHO_COMMAND_REG, 1), &echoRegistersB[ECHO_COMMAND_REG], 1);
  			  }else
  			  {
  			    echoRegisters[ECHO_COMMAND_REG] |= ECHO_BIT_MODE;
  			    Echo_Config(BUILDCMD(CMD_ONE, ECHO_COMMAND_REG, 1), &echoRegisters[ECHO_COMMAND_REG], 1);
  			  }
  			#else
  			  echoRegistersB[ECHO_COMMAND_REG] |= ECHO_BIT_MODE;  /* Only bank B is active */
  			  Echo_Config(BUILDCMD(CMD_ONE, ECHO_COMMAND_REG, 1), &echoRegistersB[ECHO_COMMAND_REG], 1);
  			#endif  
			#endif
		
			Echo_ExitConfig();

			ECHO_SPIC1 &= ~ECHO_SPIC1_SPE;		/* Disable MOSI so timer can drive it	*/
			ECHO_SEB = 0;						/* Enable Echo's interface				*/

			/* Set up driver next state													*/
			if (_ECHO_MODE_VALUE == ECHO_OOK)
			{	
				bitCounter = _ECHO_PREAMBLE_FSK00uS;
				echoDriverNextState = ECHO_PREAMBLE_OOK;
			}else
			{
				bitCounter = 4;
				echoDriverNextState = ECHO_PREAMBLE_FSK;
			}

			break;



		/*- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
		/* Preamble 1: '1' NRZ > 200 uSec, enter with bitCounter set to # bits in 200uS	*/
		case ECHO_PREAMBLE_OOK:
			ECHO_TPMxCxSC = ECHO_SET_ON_COMPARE;

			if (--bitCounter == 0) {
				echoDriverNextState = ECHO_PREAMBLE_FSK;
				#ifdef ECHO_PLUS_MODE
  				if(_ECHO_MODE_DSREF == 0) bitCounter = 1; 
  				else bitCounter = 4;
  			#else
  				#if _ECHO_MODE_DSREF == 0
  					bitCounter = 1;
  				#else
  					bitCounter = 4;
  				#endif
				#endif
			}

			break;


		/*- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
		/* Preamble 2: 0's (Manchester) to settle data slicer reference					*/
		/* Enter with bitCounter set to # 0's to send									*/
		case ECHO_PREAMBLE_FSK:
			ECHO_TPMxCxSC = ECHO_PWM_ZERO;

			if (--bitCounter == 0) {
				if (tx_config.Bits.IDHeaderSelect == 0) {
					data = (echoTxDataPtr[2]) << (8-_ECHO_ID_LENGTH);
					bitCounter = _ECHO_ID_LENGTH;
					echoDriverNextState = ECHO_SENDID;
				}
				else {
					data = _ECHO_HEADER_VALUE << (8-_ECHO_HEADER_LENGTH);
					bitCounter = _ECHO_HEADER_LENGTH;
					echoDriverNextState = ECHO_SENDHEADER;
				}
			}

			break;


		/*- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
		/* SendID: Enter with bitCounter set to # bits to send and data set to the ID	*/
		/* value left justified in the byte												*/
		case ECHO_SENDID:
			if ((data & 0x80) == 0) {
				ECHO_TPMxCxSC = ECHO_PWM_ZERO;
			}
			else {
				ECHO_TPMxCxSC = ECHO_PWM_ONE;
			}

			data = data << 1;
			bitCounter--;

			if (bitCounter == 0) {
				if (tx_config.Bits.CompleteSend == 0) {		// only send a single P+ID
					echoDriverNextState = ECHO_END_NOSWITCH;
				}
				else {	// send a whole telegram: either continuous (IDRepeat) or spaced

					if (tx_config.Bits.IDRepeat == 1) {	// continuous
						if (IDCounter == 0) {
							data = _ECHO_HEADER_VALUE << (8-_ECHO_HEADER_LENGTH);
							bitCounter = _ECHO_HEADER_LENGTH;
							echoDriverNextState = ECHO_SENDHEADER;
						}
						else {
							IDCounter--;
							data = (echoTxDataPtr[2]) << (8-_ECHO_ID_LENGTH);
							bitCounter = _ECHO_ID_LENGTH;
							echoDriverNextState = ECHO_SENDID;
						}
					}
					else { // IDRepeat == 0 i.e. spaced
						bitCounter = echoTxDataPtr[1];	/* Spacing bit-times from buf	*/
						echoDriverNextState = ECHO_WAIT;
					}
				}
			}

			break;


		/*- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
		/* SendHeader: Enter with bitCounter set to # bits to send and data set to the	*/
		/* header value left justified in the byte										*/
		case ECHO_SENDHEADER:
			if ((data & 0x80) == 0) {
				ECHO_TPMxCxSC = ECHO_PWM_ZERO;
			}
			else {
				ECHO_TPMxCxSC = ECHO_PWM_ONE;
			}

			data = data << 1;
			bitCounter--;

			if (bitCounter == 0) {    /* Send length */
				data = echoTxDataPtr[3] & ~ECHO_BUFFER_FULL;	/* Remove full bit from	*/
				bitCounter = 8;									/* length 				*/
				
				EchoCalcChecksum(data);						/* Length is part of chksum	*/
				
				echoTxDataPtr+=4;							/* Pointer to start of data	*/
				echoDriverNextState = ECHO_SENDDATA;
			}

			break;


		/*- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
		/* WAIT: Wait for a number of bit-times before resending P+ID. Ends when the	*/
		/* repeat counter has reached zero: at this point, when reentering the preamble	*/
		/* sending states, tx_config.Bits.IDHeaderSelect is set 1 to send header/data.	*/
		/* Enter with bitCounter set to number of bit-times spacing between P+ID		*/
		/* sequences and IDCounter set to number of P+ID's to repeat for. */
		case ECHO_WAIT:
			ECHO_TPMxCxSC = ECHO_CLEAR_ON_COMPARE;
			ECHO_TPMxCxVH = _ECHO_MODH;
			ECHO_TPMxCxVL = _ECHO_MODL;

			if (bitCounter == 0) {
				
				if (IDCounter == 0) {	// no more repeats
					tx_config.Bits.IDHeaderSelect = 1;
				}

				IDCounter--;							/* One less P+ID to send		*/
				
				  if(_ECHO_MODE_VALUE == ECHO_OOK)
				  {
				    echoDriverNextState = ECHO_PREAMBLE_OOK;
  					bitCounter = _ECHO_PREAMBLE_FSK00uS;
				  }else
				  {
				    echoDriverNextState = ECHO_PREAMBLE_FSK;
  					bitCounter = 4;
				  }
      
				ECHO_TPMxCxVH = _ECHO_COMH;
				ECHO_TPMxCxVL = _ECHO_COML;
			}
			else {
				bitCounter--;
			}

			break;



		/*- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
		/* SendData: Enter with data set to first byte to send, bitCounter set # of		*/
		/* bits in data, byteCounter set to the # of bytes remaining in the buffer and	*/
		/* dataPtr pointing to the buffer												*/
		case ECHO_SENDDATA:
			if ((data & 0x80) == 0) {
				ECHO_TPMxCxSC = ECHO_PWM_ZERO;
			}
			else {
				ECHO_TPMxCxSC = ECHO_PWM_ONE;
			}

			data = data << 1;
			bitCounter--;

			if (bitCounter == 0) {
				if (byteCounter == 0) {
					bitCounter = 8;
					echoChecksum = ~echoChecksum;
					echoDriverNextState = ECHO_SENDCHECKSUM;
				}
				else {
					bitCounter = 8;
					data = *echoTxDataPtr++;
					EchoCalcChecksum(data);
					byteCounter--;
				}
			}

			break;



		/*- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
		/* SendChecksum: Enter with echoChecksum set to the checksum and bitCounter set */
		/* to # of bits in checksum														*/
		case ECHO_SENDCHECKSUM:
			if ((echoChecksum & 0x80) == 0) {
				ECHO_TPMxCxSC = ECHO_PWM_ZERO;
			}
			else {
				ECHO_TPMxCxSC = ECHO_PWM_ONE;
			}

			echoChecksum = echoChecksum << 1;
			bitCounter--;

			if (bitCounter == 0) {
				echoDriverNextState = ECHO_EXTRA_BIT;
			}

			break;


		/*- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
		/* Extra Bit: Accomodate a "feature' of the data manager units and maintain		*/
		/* compatibility with Tango/Romeo												*/
		case ECHO_EXTRA_BIT:
			ECHO_TPMxCxSC = ECHO_PWM_ZERO;
			echoDriverNextState = ECHO_EOM_1;
			break;


		/*- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
		/* EOM 1: First constant low bit-time											*/
		case ECHO_EOM_1:
			ECHO_TPMxCxSC = ECHO_CLEAR_ON_COMPARE;
			echoDriverNextState = ECHO_EOM_2;
			break;

		/*- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
		/* EOM 2: Second constant low bit-time											*/
		/* If a complete send, then check for a queued message and possibly start		*/
		/* sending that. Otherwise switch back to Rx.									*/
		case ECHO_EOM_2:
			ECHO_TPMxCxSC = ECHO_CLEAR_ON_COMPARE;
			
			if (tx_config.Bits.CompleteSend == 1 && tx_config.Bits.IDRepeat == 0) {
				/* A complete telegram send always uses the internal buffer				*/
				echoInternalTxBuffer[3] &= ~ECHO_BUFFER_FULL;	/* Clear buf full flag	*/

				if ((echoTransmitBuffer[3] & ECHO_BUFFER_FULL) != 0) {	// queued msg
					/* Reset state machine to send ID									*/
					tx_config.Bits.IDHeaderSelect = 0;

					/* Set up data, length, checksum etc								*/
					EchoCopyTxBuf();

					echoTxDataPtr = echoInternalTxBuffer;
					byteCounter = echoTxDataPtr[3] & ~ECHO_BUFFER_FULL;
					echoChecksum = echoTxDataPtr[2];	/* Start with ID				*/
					IDCounter = echoTxDataPtr[0];		/* Number of P+ID repeats		*/

					/* Set up driver next state											*/
					  if(_ECHO_MODE_VALUE == ECHO_OOK)
					  {
					    echoDriverNextState = ECHO_PREAMBLE_OOK;
  						bitCounter = _ECHO_PREAMBLE_FSK00uS;
					  }else
					  {
					    echoDriverNextState = ECHO_PREAMBLE_FSK;
  						bitCounter = 4;
					  }
				}
				else {	// no queued msg
					echoDriverNextState = ECHO_SWITCHBACK;
				}
			}
			else {
				echoTransmitBuffer[3] &= ~ECHO_BUFFER_FULL;		/* Clear buf full flag	*/
				echoDriverNextState = ECHO_SWITCHBACK;
			}
			break;


		/*- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
		/* Switch Back: Initiate the switch back to Rx mode, then go to delay for 500uS	*/
		case ECHO_SWITCHBACK:
			/* Turn off channel pin to let SPI config work								*/
			ECHO_TPMxCxSC = ECHO_TIMER_PIN_OFF;

			/* Turn off the PA and turn on the LNA										*/
			#ifdef ECHO_ENABLELNA
				ECHO_ENABLELNA = 1;
			#endif

			#ifdef ECHO_ENABLEPA
				ECHO_ENABLEPA = 0;
			#endif
      
      /* Switch to Rx mode                           */
      Echo_EnterConfig();															
			#ifdef ECHO_MODE
			  echoRegisters[ECHO_COMMAND_REG] &= ~ECHO_BIT_MODE;
			  Echo_Config(BUILDCMD(CMD_ONE, ECHO_COMMAND_REG, 1), &echoRegisters[ECHO_COMMAND_REG], 1);
			#else
			  #if ECHO_BANK_A_EN == 1
  			  if(echoBankStatus & ECHO_ST_BANK_B)
  			  {
  			    echoRegistersB[ECHO_COMMAND_REG] &= ~ECHO_BIT_MODE;
  			    Echo_Config(BUILDCMD(CMD_ONE, ECHO_COMMAND_REG, 1), &echoRegistersB[ECHO_COMMAND_REG], 1);
  			   
  			    #if ECHO_AUTO_SWITCH == 1     /* Auto switch function to both banks mode */
  			      echoRegisters[ECHO_RXONOFF_REG] |= ECHO_BIT_BANK;
  			      p_echoBank = (tSWITCH_BANK_INFO*) &echoBankAConst;
  			      echoBankStatus &= ~ECHO_ST_BANK_B;
  			      echoBankStatus |= ECHO_ST_BANK_BOTH;  			      
  			      Echo_Config(BUILDCMD(CMD_ONE, ECHO_RXONOFF_REG, 1), &echoRegisters[ECHO_RXONOFF_REG], 1);
  			      status.Bits.SwitchMode = 1;
  			    #endif
  			  }else
  			  {
  			    echoRegisters[ECHO_COMMAND_REG] &= ~ECHO_BIT_MODE;
  			    Echo_Config(BUILDCMD(CMD_ONE, ECHO_COMMAND_REG, 1), &echoRegisters[ECHO_COMMAND_REG], 1);
  			    
  			    #if ECHO_AUTO_SWITCH == 1            /* Auto switch function to both banks mode */
  			      echoRegistersB[ECHO_RXONOFF_REG] |= ECHO_BIT_BANK;
  			      p_echoBank = (tSWITCH_BANK_INFO*) &echoBankBConst;
  			      echoBankStatus &= ~ECHO_ST_BANK_A;
  			      echoBankStatus |= ECHO_ST_BANK_BOTH;  			      
  			      Echo_Config(BUILDCMD(CMD_ONE, ECHO_RXONOFF_REG, 1), &echoRegistersB[ECHO_RXONOFF_REG], 1);
  			      p_echoBank = (tSWITCH_BANK_INFO*) &echoBankAConst;
  			      status.Bits.SwitchMode = 1;
  			    #endif
  			  }
  			#else
  			  echoRegistersB[ECHO_COMMAND_REG] &= ~ECHO_BIT_MODE;
  			  Echo_Config(BUILDCMD(CMD_ONE, ECHO_COMMAND_REG, 1), &echoRegistersB[ECHO_COMMAND_REG], 1);
  			#endif  
			#endif
			
			Echo_ExitConfig();	
	
			ECHO_SPIC1 &= ~ECHO_SPIC1_SPE;				/* Keep SPI disabled			*/

			status.Bits.Tx = 0;
			status.Bits.ModeSwitchDelay = 1;

			bitCounter = _ECHO_500uS_DELAY;
			echoDriverNextState = ECHO_TXRX_SWITCH_DELAY;
			ECHO_TPMxCxSC = ECHO_CLEAR_ON_COMPARE;
			break;


		/*- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
		/* TxRx Mode Switch delay: wait for 500uS then end								*/
		case ECHO_TXRX_SWITCH_DELAY:
			ECHO_TPMxCxSC = ECHO_CLEAR_ON_COMPARE;

			if (--bitCounter == 0) {
				status.Bits.Busy = 0;
				status.Bits.Mode = 0;
				status.Bits.ModeSwitchDelay = 0;

				ECHO_TPMxCxSC &= ~ECHO_TPMxCxSC_CHnIE;	/* Turn off channel pin			*/
				ECHO_TPMxCxSC = ECHO_TIMER_PIN_OFF;		/* Turn off channel pin			*/

				
         #ifdef __HCS08__                          /* Code for HCS08 mcu's */
          ECHO_SPIC1 |= ECHO_SPIC1_SPIE;			/* Enable SPI interrupts		*/
				  ECHO_SPIC1 |= ECHO_SPIC1_SPE;			/* Reenable SPI and MOSI		*/
					#if ECHO_TIMER_DISABLE == 1
					  ECHO_TPMxSC &= ECHO_TIMER_OFF;
					#endif  
         #else
          /* HC08 family MCUs code */
          #if ECHO_TIMER_DISABLE == 1
            ECHO_TPMxSC |= ECHO_TIMER_OFF;
          #endif
          ECHO_SPIC1 = 0x8a;            /* Enable SPI slave and SPI Interrupts */
         #endif

				ECHO_SEB = 0;							/* Let Echo take MOSI pin back	*/

				echoDriverNextState = ECHO_READY;
			}

			break;

		/*- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
		/* End No Switch: Stop the transmission, do not switch back to Rx				*/
		/* Can't jump straight to end because End turns off the timer which occurs		*/
		/* immediately, and therefore does not allow the last bit to finish sending.	*/
		/* Also, the output is cleared so next time timer is turned on, it starts low.	*/
		case ECHO_END_NOSWITCH:
			ECHO_TPMxCxSC = ECHO_CLEAR_ON_COMPARE;		/* Output low if timer not off	*/
			ECHO_TPMxCxVH = _ECHO_MODH;
			ECHO_TPMxCxVL = _ECHO_MODL;
			echoDriverNextState = ECHO_END;
			break;

		/*- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
		/* End: Stop the transmission, do not switch back to Rx							*/
		case ECHO_END:
			status.Bits.Busy = 0;
			status.Bits.Tx = 0;

			ECHO_TPMxCxSC &= ~ECHO_TPMxCxSC_CHnIE;		/* Turn off channel interrupts	*/
			ECHO_TPMxCxSC = ECHO_TIMER_PIN_OFF;			/* Turn off channel pin			*/

			#if ECHO_TIMER_DISABLE == 1
        #ifdef __HCS08__                            /* Code for HCS08 mcu's */
				 ECHO_TPMxSC &= ECHO_TIMER_OFF;
	    	#else
		     /* HC08 family MCUs code */
		     ECHO_TPMxSC |= ECHO_TIMER_OFF;
    		#endif

			#endif

			echoDriverNextState = ECHO_READY;
			break;


		/*- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
		/* Config Delay: 1.2ms delay when Echo enters config mode.						*/
		/* Delays before executing a command on Echo to change its configuration.		*/
		/* Enter with bitCounter = # of bit-times in 1.2ms								*/
		/* Echo_EnterConfig should be called prior to entering this state				*/
		case ECHO_CONFIG_DELAY:
			ECHO_TPMxCxSC = ECHO_COMPARE_NO_OUTPUT;

			if (--bitCounter == 0) {
				echoDriverNextState = ECHO_DO_CONFIG;
			}

			break;

		/*- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
		/* Do Config: Send a command to Echo to r/w registers.							*/
		/* Enter with echoCommand = the command, echoReg.Ptr = the buffer to read to or	*/
		/* write from and echoRegCounter = the # of registers to r/w.					*/
		case ECHO_DO_CONFIG:
			ECHO_TPMxCxSC = ECHO_COMPARE_NO_OUTPUT;

			/* We may have been in Tx mode: turn off the PA and turn on the LNA			*/
			#ifdef ECHO_ENABLELNA
				ECHO_ENABLELNA = 1;
			#endif

			#ifdef ECHO_ENABLEPA
				ECHO_ENABLEPA = 0;
			#endif
 	
			#ifdef ECHO_MODE    /* Normal Echo mode - no bank switch or other features */
			  /* Do the actual configuration required										*/
			  Echo_Config(echoCommand, echoReg.Ptr, echoRegCounter);
			  /* Switch to Rx mode														*/
			  echoRegisters[ECHO_COMMAND_REG] &= ~ECHO_BIT_MODE;
			  Echo_Config(BUILDCMD(CMD_ONE, ECHO_COMMAND_REG, 1), &echoRegisters[ECHO_COMMAND_REG], 1);
			  #if ECHO_LVD_ENABLE == 1
          Echo_Config(BUILDCMD(CMD_ONE, ECHO_CONFIG3_REG, ECHO_REG_READ),&echoRegisters[ECHO_CONFIG3_REG] , 1);
  		  #endif
			#else
			                           /* Echo Plus mode  - bank switching and writing to both reg bank */
			  #if ECHO_BANK_A_EN == 1
			    // Both banks are enabled
  			  if(echoBankStatus & ECHO_ST_BANK_SWITCH) /* Switch banks??? */
  			    {
  			      
  			      if(echoBankStatus & ECHO_ST_BANK_WR_B) /* Write to bank B */
  			      {
  			        p_echoBank = (tSWITCH_BANK_INFO*) &echoBankBConst;
  			        Echo_Config(BUILDCMD(CMD_ONE, ECHO_RXONOFF_REG, 1), &echoRegistersB[ECHO_RXONOFF_REG], 1);
  			        if((echoBankStatus & (ECHO_ST_BANK_A | ECHO_ST_BANK_BOTH))) 
  			          p_echoBank = (tSWITCH_BANK_INFO*) &echoBankAConst;
  			        echoBankStatus &= ~ECHO_ST_BANK_WR_B;
  			      }
  			      
  			      if(echoBankStatus & ECHO_ST_BANK_WR_A)  /* Write to bank A */
  			      {
  			        p_echoBank = (tSWITCH_BANK_INFO*) &echoBankAConst;
  			        Echo_Config(BUILDCMD(CMD_ONE, ECHO_RXONOFF_REG, 1), &echoRegisters[ECHO_RXONOFF_REG], 1);
  			        echoBankStatus &= ~ECHO_ST_BANK_WR_A;
  			        if(echoBankStatus & ECHO_ST_BANK_B) 
  			          p_echoBank = (tSWITCH_BANK_INFO*) &echoBankBConst;
  			      }
  			      echoBankStatus &= ~ECHO_ST_BANK_SWITCH;
  			      /* Write to both bank in one time ? */
  			    }else if((echoBankStatus & (ECHO_ST_BANK_WR_A | ECHO_ST_BANK_WR_B)) == (ECHO_ST_BANK_WR_A | ECHO_ST_BANK_WR_B))
  			      {  			        
			          p_echoBank = (tSWITCH_BANK_INFO*) &echoBankBConst;
			          
			          Echo_Config(echoCommand, &echoRegistersB[echoReg.Offset], echoRegCounter);
			          p_echoBank = (tSWITCH_BANK_INFO*) &echoBankAConst;
			          Echo_Config(echoCommand, &echoRegisters[echoReg.Offset], echoRegCounter);			          
  			        echoBankStatus &= ~(ECHO_ST_BANK_WR_A | ECHO_ST_BANK_WR_B);
  			        if(echoBankStatus & ECHO_ST_BANK_B) 
  			          p_echoBank = (tSWITCH_BANK_INFO*) &echoBankBConst;  			      
  			      }else 
  			    
  			  {  /* Normal function for config registers */
  			  if(echoBankStatus & ECHO_ST_BANK_BOTH)
    			  {
    			    status.Bits.Error = 1;
    			    /* Switch to Rx mode                                       */
  			      if(echoRegisters[ECHO_COMMAND_REG] & ECHO_BIT_MODE)
  			      {
  			        echoRegisters[ECHO_COMMAND_REG] &= ~ECHO_BIT_MODE;
  			        Echo_Config(BUILDCMD(CMD_ONE, ECHO_COMMAND_REG, 1), &echoRegisters[ECHO_COMMAND_REG], 1);
  			        
  			      }
  			      if(echoRegistersB[ECHO_COMMAND_REG] & ECHO_BIT_MODE)
  			      {
  			        echoRegistersB[ECHO_COMMAND_REG] &= ~ECHO_BIT_MODE;
  			        p_echoBank = (tSWITCH_BANK_INFO*) &echoBankBConst;
  			        Echo_Config(BUILDCMD(CMD_ONE, ECHO_COMMAND_REG, 1), &echoRegistersB[ECHO_COMMAND_REG], 1);
  			        p_echoBank = (tSWITCH_BANK_INFO*) &echoBankAConst;
  			      }
  			      #if ECHO_LVD_ENABLE == 1
  			        Echo_Config(BUILDCMD(CMD_ONE, ECHO_CONFIG3_REG, ECHO_REG_READ),&echoRegisters[ECHO_CONFIG3_REG] , 1);
  			        echoRegistersB[ECHO_CONFIG3_REG] &= ~ECHO_LVDS_BIT;
  			        echoRegistersB[ECHO_CONFIG3_REG] |= ( ECHO_LVDS_BIT & echoRegisters[ECHO_CONFIG3_REG]);
  			      #endif  
  			            
    			  }else if(echoBankStatus & ECHO_ST_BANK_B)
    			  { // Bank B is select
        			/* Do the actual configuration required										*/
        			Echo_Config(echoCommand, echoReg.Ptr, echoRegCounter);
      			  /* Switch to Rx mode                                       */
  			      echoRegistersB[ECHO_COMMAND_REG] &= ~ECHO_BIT_MODE;
  			      Echo_Config(BUILDCMD(CMD_ONE, ECHO_COMMAND_REG, 1), &echoRegistersB[ECHO_COMMAND_REG], 1);
              #if ECHO_LVD_ENABLE == 1
                Echo_Config(BUILDCMD(CMD_ONE, ECHO_CONFIG3_REG, ECHO_REG_READ),&echoRegistersB[ECHO_CONFIG3_REG] , 1);
    			      echoRegisters[ECHO_CONFIG3_REG] &= ~ECHO_LVDS_BIT;
    			      echoRegisters[ECHO_CONFIG3_REG] |= ( ECHO_LVDS_BIT & echoRegistersB[ECHO_CONFIG3_REG]);
    			    #endif
    			  }else
    			  { // Bank A is select
    			   
  		        /* Do the actual configuration required										*/
    			    Echo_Config(echoCommand, echoReg.Ptr, echoRegCounter);
      			  /* Switch to Rx mode                                       */
  			      echoRegisters[ECHO_COMMAND_REG] &= ~ECHO_BIT_MODE;
  			      Echo_Config(BUILDCMD(CMD_ONE, ECHO_COMMAND_REG, 1), &echoRegisters[ECHO_COMMAND_REG], 1);
  			      #if ECHO_LVD_ENABLE == 1
  			        Echo_Config(BUILDCMD(CMD_ONE, ECHO_CONFIG3_REG, ECHO_REG_READ),&echoRegisters[ECHO_CONFIG3_REG] , 1);
  			        echoRegistersB[ECHO_CONFIG3_REG] &= ~ECHO_LVDS_BIT;
  			        echoRegistersB[ECHO_CONFIG3_REG] |= ( ECHO_LVDS_BIT & echoRegisters[ECHO_CONFIG3_REG]);
  			      #endif
    			  }
    			  
    			  Echo_Config(BUILDCMD(CMD_ONE, ECHO_CONFIG3_REG, ECHO_REG_READ),&echoRegisters[ECHO_CONFIG3_REG] , 1);
    			  
  			  }
  			#else
  			  /* Do the actual configuration required										*/
  			  Echo_Config(echoCommand, echoReg.Ptr, echoRegCounter);
  			  /* Switch to Rx mode                                    */
  			  echoRegistersB[ECHO_COMMAND_REG] &= ~ECHO_BIT_MODE;
  			  Echo_Config(BUILDCMD(CMD_ONE, ECHO_COMMAND_REG, 1), &echoRegistersB[ECHO_COMMAND_REG], 1);
  			  #if ECHO_LVD_ENABLE == 1
            Echo_Config(BUILDCMD(CMD_ONE, ECHO_CONFIG3_REG, ECHO_REG_READ),&echoRegistersB[ECHO_CONFIG3_REG] , 1);
  		    #endif
  			#endif  
			#endif

	
			Echo_ExitConfig();

			ECHO_SPIC1 &= ~ECHO_SPIC1_SPE;				/* Keep SPI disabled			*/

			status.Bits.ModeSwitchDelay = 1;

      

			bitCounter = _ECHO_500uS_DELAY;
			echoDriverNextState = ECHO_TXRX_SWITCH_DELAY;
			break;


				/*- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
		/* (Rx States): Decrement the counter and check for timeout							*/
		case ECHO_READ_DATA:
			ECHO_TPMxCxSC = ECHO_COMPARE_NO_OUTPUT;
			
			bitCounter--;
			if (bitCounter == 0) { 	/* Time out - back to ready state */
				status.Bits.Error = 1;
				status.Bits.Timeout = 1;

				status.Bits.Busy = 0;
				status.Bits.Rx = 0;
				echoDriverNextState = ECHO_READY;

				ECHO_TPMxCxSC &= ~ECHO_TPMxCxSC_CHnIE;
				ECHO_TPMxCxSC = ECHO_TIMER_PIN_OFF;

				#if ECHO_TIMER_DISABLE == 1
				 #ifdef __HCS08__            /* Code for HCS08 mcu's  */
					ECHO_TPMxSC &= ECHO_TIMER_OFF;
				 #else
			    /* HC08 family MCUs code */
			    ECHO_TPMxSC |= ECHO_TIMER_OFF;
			   #endif
			
				#endif
			}
			
			break;
		
		
		/*- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
		/* (Rx States): Decrement the counter and check for timeout for LAST BYTE							*/	
		case ECHO_LAST_BYTE:	
      ECHO_TPMxCxSC = ECHO_COMPARE_NO_OUTPUT;
			bitCounter--;
			if (bitCounter == 0)
			  Echo_EndReceiveMsg();
			break;	


		/*- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
		/* Oh dear something went wrong!												*/
		default:
			status.Bits.Error = 1;
			break;
	}
}


/**---------------------------------------------------------------------------------------
 * SPI Rx interrupts occur from Echo when data is received. It is processed in here.
 */
interrupt void Echo_RxSPI_Interrupt(void) {

	byte temp;

	temp = ECHO_SPIS;							/* Read SPRF flag set					*/
	temp = ECHO_SPID;							/* Read data reg and clear SPRF flag	*/

	switch (echoDriverNextState) {

		/*- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
		/* Ready for a message - the first byte received will be the length				*/
		case ECHO_READY:
			if (temp <= 0 || temp > ECHO_MAX_DATA_SIZE) {	/* Length error!			*/
				ECHO_CONFB = 0;						/* Reset Echo state machine			*/
				ECHO_CONFB = 1;
				break;
			}
      #ifdef ECHO_MODE
			  echoChecksum = _ECHO_ID_VALUE;			/* Checksum includes ID 	 */
      #else
        echoChecksum = 0;                   /* ID will add to checksum at a last byte */
      #endif
      
      EchoCalcChecksum(temp);               	/* Checksum includes length	 */
			echoRxDataPtr = echoInternalRxBuffer;	/* Where to put the message			*/
			echoDataLength = temp+1;				/* Add one for checksum				*/

			echoRxDataPtr[1] = temp;				/* Store length in buffer			*/
			echoRxDataPtr += 2;						/* Move to start of data			*/

			status.Bits.Busy = 1;
			status.Bits.Rx   = 1;
			echoDriverNextState = ECHO_READ_DATA;

			#if ECHO_BANK_A_EN == 1 && ECHO_BANK_B_EN == 1
			  if(echoBankStatus & ECHO_ST_BANK_BOTH)
			  {
  			  #if ECHO_FASTER_BANK == ECHO_BANK_A
  			     bitCounter = ECHO_TIMEOUT_TIME;
  			  #else
  			     bitCounter = ECHO_RX_TIMEOUT_BITS;
  			  #endif
  			}  
			#else
			  bitCounter = ECHO_RX_TIMEOUT_BITS;		/* Reset timeout					*/
			#endif
			
			Echo_KickOffTimer();					/* Start timeout countdown			*/
			
			#ifdef ECHO_RSSIC
  			/* Start RSSI conversion if enabled											*/
  			if (status.Bits.RSSI_Enabled == 1) {
  				ECHO_RSSIC = 1;           
  			}
			#endif
			
			break;


		/*- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
		/* Read the data of a message													*/
		case ECHO_READ_DATA:
			#if ECHO_BANK_A_EN == 1 && ECHO_BANK_B_EN == 1
			  if(echoBankStatus & ECHO_ST_BANK_BOTH)
			  {
  			  #if ECHO_FASTER_BANK == ECHO_BANK_A
  			     bitCounter = ECHO_TIMEOUT_TIME;
  			  #else
  			     bitCounter = ECHO_RX_TIMEOUT_BITS;
  			  #endif
  			}  
			#else
			  bitCounter = ECHO_RX_TIMEOUT_BITS;		/* Reset timeout					*/
			#endif
			EchoCalcChecksum(temp);
			*echoRxDataPtr++ = temp;
			echoDataLength--;
			if (echoDataLength == 0) {
				echoDriverNextState = ECHO_LAST_BYTE;
			}
			break;


		/*- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
		/* Got the last byte of a message - discard as it contains 'extra bit'			*/
		case ECHO_LAST_BYTE:
			Echo_EndReceiveMsg();
			break;


		/*- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
		/* Reading the RSSI from the SPI												*/
		case ECHO_READ_RSSI_1:
			ECHO_SPID = 0;
			
			echoDriverNextState = ECHO_READ_RSSI_2;
			break;

		/*- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
		/* Reading the RSSI from the SPI												*/
		case ECHO_READ_RSSI_2:
			
			echoEmptyPtr[0] &= 0b11000000;				/* Clear the RSSI bits			*/
			echoEmptyPtr[0] |= (temp & 0x0F);			/* Copy in bottom four bits		*/
			echoEmptyPtr[0] += ((temp & 0xF0) >> 4);	/* Add top four bits for RSSI	*/
			echoEmptyPtr[0] |= ECHO_RSSI_BIT;			/* Indicate RSSI measured		*/

			echoEmptyPtr[1] |= ECHO_BUFFER_FULL;		/* Signal buffer full			*/

			echoEmptyPtr += ECHO_MAX_DATA_SIZE+2;
			if (echoEmptyPtr >= (echoRxBuffer+sizeof(echoRxBuffer))) {
				echoEmptyPtr = echoRxBuffer;
			}

			Echo_ExitConfig();
			ECHO_SEB = 0;

			status.Bits.Busy = 0;
			status.Bits.Rx = 0;

			echoDriverNextState = ECHO_READY;
			break;

#ifdef ECHO_PLUS_MODE
	/*- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
		/* Reading the BANK bit from the SPI												*/
		case ECHO_READ_BANK_1:
			ECHO_SPID = 0;
			echoDriverNextState = ECHO_READ_BANK_2;
			break;

		/*- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
		/* Reading the BANK bit from the SPI												*/
		case ECHO_READ_BANK_2:
			echoEmptyPtr[0] &= 0b10111111;				/* Clear the Bank bit in buffer			*/
			if((status.Bits.SwitchMode) && (ECHO_STROBE_DDR == 0))
			  echoEmptyPtr[0] |= (temp & ECHO_BIT_BANKS)? ECHO_BANK_B : ECHO_BANK_A;			/* Set the bank value	*/
			else
			  echoEmptyPtr[0] |= (temp & ECHO_BIT_BANKS)? ECHO_BANK_A : ECHO_BANK_B;			/* Set the bank value	*/

      /* Stop RSSI conversion if enabled											*/
			if (status.Bits.RSSI_Enabled == 1) {
				#ifdef ECHO_RSSIC
				  ECHO_RSSIC = 0;
				#endif  
        ECHO_CONFB = 1;       
        ECHO_CONFB = 0;
				ECHO_SPID = BUILDCMD(CMD_ONE, ECHO_RSSI_REG, 0);
				echoDriverNextState = ECHO_READ_RSSI_1;
			}
		
		  /* Select right ID which has to add to checksum */
			#if ECHO_BANK_A_EN ==1 && ECHO_BANK_B_EN == 1
			  if(echoEmptyPtr[0] & ECHO_BANK_A) 
			    p_echoBank = (tSWITCH_BANK_INFO*) &echoBankAConst;
			  else
			    p_echoBank = (tSWITCH_BANK_INFO*) &echoBankBConst;
			  EchoCalcChecksum(_ECHO_ID_VALUE); /* Checksum includes ID 	*/
			  if(echoBankStatus & ECHO_ST_BANK_B)
			    p_echoBank = (tSWITCH_BANK_INFO*) &echoBankBConst;
			  else
			    p_echoBank = (tSWITCH_BANK_INFO*) &echoBankAConst;
			#else
			  EchoCalcChecksum(_ECHO_ID_VALUE); /* Checksum includes ID 	*/
			#endif
			
			/* Confirm the checksum														*/
			if (echoChecksum != 0xFF) {
				echoEmptyPtr[0] |= ECHO_CHECKSUM_ERROR;
			}
			else {
				echoEmptyPtr[0] &= ~ECHO_CHECKSUM_ERROR;
			}
			
      /* Signal buffer full and move on the empty pointer only when completely	*/
			/* finished processing a message - that includes the RSSI as well			*/
			if (status.Bits.RSSI_Enabled == 0) {
				echoEmptyPtr[0] &= 0b11000000;			/* Clear the RSSI bits			*/
				echoEmptyPtr[1] |= ECHO_BUFFER_FULL;

				echoEmptyPtr += ECHO_MAX_DATA_SIZE+2;
				if (echoEmptyPtr >= (echoRxBuffer+sizeof(echoRxBuffer))) {
					echoEmptyPtr = echoRxBuffer;
				}
        
        Echo_ExitConfig();
			  ECHO_SEB = 0;
				
				status.Bits.Busy = 0;
				status.Bits.Rx = 0;
			  echoDriverNextState = ECHO_READY;
			}	
			break;

#endif

		/*- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
		/* Oh dear something went wrong!												*/
		default:
			status.Bits.Error = 1;
			break;
	}
}

/**---------------------------------------------------------------------------------------
 * Internal function which is used for finalization received messages. Should be call from two 
 * event: 1 - from SPI interrupt (ECHO send last dummy byte)
 *        2 - from timeout (ECHO not sent last dummy byte)
 */

void Echo_EndReceiveMsg(void)
{
			byte i;
			echoDriverNextState = ECHO_READY;
			/* Stop the timeout countdown												*/
			ECHO_TPMxCxSC &= ~ECHO_TPMxCxSC_CHnIE;
			ECHO_TPMxCxSC = ECHO_TIMER_PIN_OFF;
			#if ECHO_TIMER_DISABLE == 1
    	 #ifdef __HCS08__                      /* Code for HCS08 mcu's    */		
				ECHO_TPMxSC &= ECHO_TIMER_OFF;
       #else
        /* HC08 family MCUs code */
        ECHO_TPMxSC |= ECHO_TIMER_OFF;
       #endif
			#endif
                                                          
    
			/* Check for a free buffer													*/
			if ((echoEmptyPtr[1] & ECHO_BUFFER_FULL) != 0) {
				status.Bits.Overrun = 1;
				status.Bits.Busy = 0;
				status.Bits.Rx = 0;
				
			}else {

			/* There is a free buffer, so clear Overrun flag							*/
			status.Bits.Overrun = 0;

      /* Copy the received message to external buffer								*/
			for(i=1; i<ECHO_MAX_DATA_SIZE+2; i++) {
				echoEmptyPtr[i] = echoInternalRxBuffer[i];
			}

			#ifdef ECHO_PLUS_MODE
			    Echo_EnterConfig();
			  	ECHO_SPIC1 |= ECHO_SPIC1_SPIE;			/* Enable SPI interrupts		*/

  				ECHO_SPID = BUILDCMD(CMD_ONE, ECHO_COMMAND_REG, ECHO_REG_READ);
  				echoDriverNextState = ECHO_READ_BANK_1;
			  
			#else
			   echoEmptyPtr[0] |= ECHO_BANK_A; // set up bank bit to BANK_A
  			/* Stop RSSI conversion if enabled											*/
  			if (status.Bits.RSSI_Enabled == 1) {
  				#ifdef ECHO_RSSIC
  				  ECHO_RSSIC = 0;
          #endif
  				Echo_EnterConfig();
  				ECHO_SPIC1 |= ECHO_SPIC1_SPIE;			/* Enable SPI interrupts		*/

  				ECHO_SPID = BUILDCMD(CMD_ONE, ECHO_RSSI_REG, 0);
  				echoDriverNextState = ECHO_READ_RSSI_1;
  			}
			
  			/* Confirm the checksum														*/
  			if (echoChecksum != 0xFF) {
  				echoEmptyPtr[0] |= ECHO_CHECKSUM_ERROR;
  			}
  			else {
  				echoEmptyPtr[0] &= ~ECHO_CHECKSUM_ERROR;
  			}
  			

  			/* Signal buffer full and move on the empty pointer only when completely	*/
  			/* finished processing a message - that includes the RSSI as well			*/
  			if (status.Bits.RSSI_Enabled == 0) {
  				echoEmptyPtr[0] &= 0b11000000;			/* Clear the RSSI bits			*/
  				echoEmptyPtr[1] |= ECHO_BUFFER_FULL;

  				echoEmptyPtr += ECHO_MAX_DATA_SIZE+2;
  				if (echoEmptyPtr >= (echoRxBuffer+sizeof(echoRxBuffer))) {
  					echoEmptyPtr = echoRxBuffer;
  				}

  				status.Bits.Busy = 0;
  				status.Bits.Rx = 0;
  			}
			#endif
    }
}
/**---------------------------------------------------------------------------------------
 * Return the status of the driver. Updates echoNextMessage pointer if another message is
 * ready and the current message buffer has been released (by clearing the buffer full
 * bit)
 */
tECHO_STATUS Echo_DriverStatus(void) {
	byte *tempPtr;
	tECHO_STATUS toReturn = status;
	
	/* echoFullPtr is either full (so it will be returned next) or empty. If it is		*/
	/* empty, the last message returned has been released so check if there is another	*/
	/* waiting in the next buffer.														*/
	if ((echoFullPtr[1] & ECHO_BUFFER_FULL) != 0) {
		echoNextMessage = echoFullPtr;
		toReturn.Bits.MsgReady = 1;

		if ((echoNextMessage[0] & ECHO_CHECKSUM_ERROR) != 0) {
			toReturn.Bits.Checksum = 1;
		}
	}
	else {
		tempPtr = echoFullPtr + ECHO_MAX_DATA_SIZE+2;
		if (tempPtr >= (echoRxBuffer+sizeof(echoRxBuffer))) {
			tempPtr = echoRxBuffer;
		}

		if ((tempPtr[1] & ECHO_BUFFER_FULL) != 0) {
			echoFullPtr = tempPtr;
			echoNextMessage = tempPtr;

			toReturn.Bits.MsgReady = 1;

			if ((echoNextMessage[0] & ECHO_CHECKSUM_ERROR) != 0) {
				toReturn.Bits.Checksum = 1;
			}
		}
	}

  #if ECHO_LVD_ENABLE == 1
    #if ECHO_BANK_B_EN == 1 && ECHO_BANK_A_EN == 0
      status.Bits.LVD = (ECHO_LVDS_BIT & echoRegistersB[ECHO_CONFIG3_REG])? 1:0 ;
    #else
      status.Bits.LVD = (ECHO_LVDS_BIT & echoRegisters[ECHO_CONFIG3_REG])? 1:0 ;
    #endif  
  #endif


	return toReturn;
}


/**---------------------------------------------------------------------------------------
 * Clear the error and timeout flags
 */
void Echo_ClearError(void) {
	status.Bits.Error = 0;
	status.Bits.Timeout = 0;
}


/**---------------------------------------------------------------------------------------
 * Take the STROBE line high (if it is under driver control). When STROBE is high, Echo
 * will always be on in Rx mode.
 */
void Echo_StrobeHigh(void) {
	#ifdef ECHO_STROBE
		ECHO_STROBE = 1;
		ECHO_STROBE_DDR = 1;
	#endif
}

/**---------------------------------------------------------------------------------------
 * Take the STROBE line low (if it is under driver control). When STROBE is low, Echo
 * will stay in Standby once entered.
 */
void Echo_StrobeLow(void) {
	#ifdef ECHO_STROBE
		ECHO_STROBE = 0;
		ECHO_STROBE_DDR = 1;
	#endif
}

/**---------------------------------------------------------------------------------------
 * Tristate the STROBE line low (if it is under driver control). When STROBE is tristated,
 * Echo's internal strobe oscillator is used (if SOE bit is 1) to control on/off time.
 */
void Echo_StrobeTriState(void) {
	#ifdef ECHO_STROBE
		ECHO_STROBE_DDR = 0;
	#endif
}


/**---------------------------------------------------------------------------------------
 * Setup and start off a delayed configuration. Schedules a 1.2ms wait before sending
 * 'echoCommand' and 'echoRegCounter' number of registers to/from 'echoReg.Ptr'.
 */
//#pragma INLINE
void Echo_StartDelayedConfig(void) {
	/* Echo will be reset back to Rx mode: make flags consistent							*/
	status.Word &= ECHO_STATUS_BANK | ECHO_STATUS_TIMEOUT | ECHO_STATUS_ERROR | ECHO_STATUS_RSSI_ENABLED | \
	ECHO_STATUS_OVERRUN | ECHO_STATUS_MSGREADY | ECHO_STATUS_ENABLED;						/* Clear all but Timeout, Error,	*/
														/* Overrun, Enabled, Msg_ready & RSSI_Enabled	*/
	status.Bits.Busy = 1;

	//ECHO_TPMxCxSC &= ~ECHO_TPMxCxSC_CHnIE;					/* in case we're transmitting	*/
	Echo_EnterConfig();

	bitCounter = _ECHO_1200uS_DELAY;
	echoDriverNextState = ECHO_CONFIG_DELAY;

	Echo_KickOffTimer();
}

/****************************************************************************************/
/* Frequency control functions - one version for OOK and one for FSK.					*/


/**---------------------------------------------------------------------------------------
 * Set frequency using the user friendly mode - specify the carrier used for OOK.
 * Carrier is a 12 bit value which can be derived from the datasheet.
 */
	void Echo_SetFreqOok(unsigned int carrier) {
		#ifdef ECHO_MODE
  		echoRegisters[ECHO_CONFIG2_REG] &= ~ECHO_BIT_FRM;
  		echoRegisters[ECHO_F1_REG] = ((carrier & 0x0F00)>>8);
  		echoRegisters[ECHO_F0_REG] =  (carrier & 0x00FF);
  		
  		echoReg.Ptr		= &echoRegisters[ECHO_CONFIG2_REG];
    #else
      #if ECHO_BANK_A_EN == 1
        if(echoBankStatus & ECHO_ST_BANK_BOTH)
        {
          echoRegistersB[ECHO_CONFIG2_REG] &= ~ECHO_BIT_FRM;
          echoRegisters[ECHO_CONFIG2_REG] &= ~ECHO_BIT_FRM;
      		echoRegistersB[ECHO_F1_REG] = ((carrier & 0x0F00)>>8);
      		echoRegisters[ECHO_F1_REG] = ((carrier & 0x0F00)>>8);
      		echoRegistersB[ECHO_F0_REG] =  (carrier & 0x00FF);
      		echoRegisters[ECHO_F0_REG] =  (carrier & 0x00FF);
          
          echoBankStatus |= ECHO_ST_BANK_WR_A | ECHO_ST_BANK_WR_B;
          echoReg.Offset = ECHO_CONFIG2_REG;
          
        }else if(echoBankStatus & ECHO_ST_BANK_B)
        {
          echoRegistersB[ECHO_CONFIG2_REG] &= ~ECHO_BIT_FRM;
      		echoRegistersB[ECHO_F1_REG] = ((carrier & 0x0F00)>>8);
      		echoRegistersB[ECHO_F0_REG] =  (carrier & 0x00FF);
      		
      		echoReg.Ptr		= &echoRegistersB[ECHO_CONFIG2_REG];  
        }else
        {
          echoRegisters[ECHO_CONFIG2_REG] &= ~ECHO_BIT_FRM;
      		echoRegisters[ECHO_F1_REG] = ((carrier & 0x0F00)>>8);
      		echoRegisters[ECHO_F0_REG] =  (carrier & 0x00FF);
      		
      		echoReg.Ptr		= &echoRegisters[ECHO_CONFIG2_REG];
        }        
      #else
        echoRegistersB[ECHO_CONFIG2_REG] &= ~ECHO_BIT_FRM;
    		echoRegistersB[ECHO_F1_REG] = ((carrier & 0x0F00)>>8);
    		echoRegistersB[ECHO_F0_REG] =  (carrier & 0x00FF);
    		
    		echoReg.Ptr		= &echoRegistersB[ECHO_CONFIG2_REG];  
      #endif
    #endif
    
		echoCommand		= BUILDCMD(CMD_EIGHT, ECHO_CONFIG2_REG, 1);
		echoRegCounter	= 8;

		Echo_StartDelayedConfig();
	}

/**---------------------------------------------------------------------------------------
 * Set frequency using the direct mode - specify the carrier and local oscillator
 * frequencies used for OOK. localOscillator and carrier0 are 12 bit values that may be
 * derived from datasheet.
 */
	void Echo_SetFreqOokNoFRM(unsigned int localOscillator, unsigned int carrier0) {
		#ifdef ECHO_MODE
  		echoRegisters[ECHO_CONFIG2_REG] |= ECHO_BIT_FRM;

  		echoRegisters[ECHO_F1_REG] = ((localOscillator & 0x0F00)>>8);
  		echoRegisters[ECHO_F0_REG] =  (localOscillator & 0x00FF);
  		
  		echoRegisters[ECHO_FT2_REG] = ((carrier0 & 0x0FF0) >> 4);
  		echoRegisters[ECHO_FT1_REG] = ((carrier0 & 0x000F) << 4);
  		
  		echoReg.Ptr		= &echoRegisters[ECHO_CONFIG2_REG];
  	#else
      #if ECHO_BANK_A_EN == 1
        if(echoBankStatus & ECHO_ST_BANK_BOTH)  /* Set up both banks */
        {
          echoRegisters[ECHO_CONFIG2_REG] |= ECHO_BIT_FRM;

      		echoRegisters[ECHO_F1_REG] = ((localOscillator & 0x0F00)>>8);
      		echoRegisters[ECHO_F0_REG] =  (localOscillator & 0x00FF);
      		
      		echoRegisters[ECHO_FT2_REG] = ((carrier0 & 0x0FF0) >> 4);
      		echoRegisters[ECHO_FT1_REG] = ((carrier0 & 0x000F) << 4);
      		
      		echoRegistersB[ECHO_CONFIG2_REG] |= ECHO_BIT_FRM;

      		echoRegistersB[ECHO_F1_REG] = ((localOscillator & 0x0F00)>>8);
      		echoRegistersB[ECHO_F0_REG] =  (localOscillator & 0x00FF);
      		
      		echoRegistersB[ECHO_FT2_REG] = ((carrier0 & 0x0FF0) >> 4);
      		echoRegistersB[ECHO_FT1_REG] = ((carrier0 & 0x000F) << 4);
      		
      		echoBankStatus |= ECHO_ST_BANK_WR_A | ECHO_ST_BANK_WR_B;
          echoReg.Offset = ECHO_CONFIG2_REG;
      		
        }else if(echoBankStatus & ECHO_ST_BANK_B)  /* Set up Bank B*/
        {
      		echoRegistersB[ECHO_CONFIG2_REG] |= ECHO_BIT_FRM;

      		echoRegistersB[ECHO_F1_REG] = ((localOscillator & 0x0F00)>>8);
      		echoRegistersB[ECHO_F0_REG] =  (localOscillator & 0x00FF);
      		
      		echoRegistersB[ECHO_FT2_REG] = ((carrier0 & 0x0FF0) >> 4);
      		echoRegistersB[ECHO_FT1_REG] = ((carrier0 & 0x000F) << 4);
      		
      		echoReg.Ptr		= &echoRegistersB[ECHO_CONFIG2_REG];  
        }else                                       /* Set up Bank A */
        {
      		echoRegisters[ECHO_CONFIG2_REG] |= ECHO_BIT_FRM;

      		echoRegisters[ECHO_F1_REG] = ((localOscillator & 0x0F00)>>8);
      		echoRegisters[ECHO_F0_REG] =  (localOscillator & 0x00FF);
      		
      		echoRegisters[ECHO_FT2_REG] = ((carrier0 & 0x0FF0) >> 4);
      		echoRegisters[ECHO_FT1_REG] = ((carrier0 & 0x000F) << 4);
      		
      		echoReg.Ptr		= &echoRegisters[ECHO_CONFIG2_REG];
        }        
      #else
    		echoRegistersB[ECHO_CONFIG2_REG] |= ECHO_BIT_FRM;

    		echoRegistersB[ECHO_F1_REG] = ((localOscillator & 0x0F00)>>8);
    		echoRegistersB[ECHO_F0_REG] =  (localOscillator & 0x00FF);
    		
    		echoRegistersB[ECHO_FT2_REG] = ((carrier0 & 0x0FF0) >> 4);
    		echoRegistersB[ECHO_FT1_REG] = ((carrier0 & 0x000F) << 4);
    		
    		echoReg.Ptr		= &echoRegistersB[ECHO_CONFIG2_REG];  
      #endif
  	#endif	
		echoCommand		= BUILDCMD(CMD_EIGHT, ECHO_CONFIG2_REG, 1);
		
		echoRegCounter	= 8;

		Echo_StartDelayedConfig();
	}



/**---------------------------------------------------------------------------------------
 * Set frequency using the user friendly mode - specify the carrier frequency and delta
 * used to calculate the actual carriers for a 0 and 1 used for FSK. The carrier is 12
 * bit, deltaF is four bits and can be derived using formulae in the datasheet.
 */
	void Echo_SetFreqFsk(unsigned int carrier, byte deltaF) {
		#ifdef ECHO_MODE
  		echoRegisters[ECHO_CONFIG2_REG] &= ~ECHO_BIT_FRM;
  		echoRegisters[ECHO_F1_REG] = ((deltaF & 0x0F)<<4) | ((carrier & 0x0F00)>>8);
  		echoRegisters[ECHO_F0_REG] =  (carrier & 0x00FF);
  		
  		echoReg.Ptr		= &echoRegisters[ECHO_CONFIG2_REG];
    #else
      #if ECHO_BANK_A_EN == 1
        if(echoBankStatus & ECHO_ST_BANK_BOTH)
        {
          echoRegisters[ECHO_CONFIG2_REG] &= ~ECHO_BIT_FRM;
      		echoRegisters[ECHO_F1_REG] = ((deltaF & 0x0F)<<4) | ((carrier & 0x0F00)>>8);
      		echoRegisters[ECHO_F0_REG] =  (carrier & 0x00FF);
      		
      		echoRegistersB[ECHO_CONFIG2_REG] &= ~ECHO_BIT_FRM;
      		echoRegistersB[ECHO_F1_REG] = ((deltaF & 0x0F)<<4) | ((carrier & 0x0F00)>>8);
      		echoRegistersB[ECHO_F0_REG] =  (carrier & 0x00FF);
      		
      		echoBankStatus |= ECHO_ST_BANK_WR_A | ECHO_ST_BANK_WR_B;
          echoReg.Offset = ECHO_CONFIG2_REG;
          
        }else if(echoBankStatus & ECHO_ST_BANK_B)
        {
          echoRegistersB[ECHO_CONFIG2_REG] &= ~ECHO_BIT_FRM;
      		echoRegistersB[ECHO_F1_REG] = ((deltaF & 0x0F)<<4) | ((carrier & 0x0F00)>>8);
      		echoRegistersB[ECHO_F0_REG] =  (carrier & 0x00FF);
      		
      		echoReg.Ptr		= &echoRegistersB[ECHO_CONFIG2_REG];  
        }else
        {
          echoRegisters[ECHO_CONFIG2_REG] &= ~ECHO_BIT_FRM;
      		echoRegisters[ECHO_F1_REG] = ((deltaF & 0x0F)<<4) | ((carrier & 0x0F00)>>8);
      		echoRegisters[ECHO_F0_REG] =  (carrier & 0x00FF);
      		
      		echoReg.Ptr		= &echoRegisters[ECHO_CONFIG2_REG];
        }        
      #else
        echoRegistersB[ECHO_CONFIG2_REG] &= ~ECHO_BIT_FRM;
    		echoRegistersB[ECHO_F1_REG] = ((deltaF & 0x0F)<<4) | ((carrier & 0x0F00)>>8);
    		echoRegistersB[ECHO_F0_REG] =  (carrier & 0x00FF);
    		
    		echoReg.Ptr		= &echoRegistersB[ECHO_CONFIG2_REG];  
      #endif

    #endif
		echoCommand		= BUILDCMD(CMD_EIGHT, ECHO_CONFIG2_REG, 1);
		
		echoRegCounter	= 8;

		Echo_StartDelayedConfig();
	}

/**---------------------------------------------------------------------------------------
 * Set frequency using the direct mode - specify the carriers for transmitting a 0 and 1
 * explicitly and the local oscillator frequency used for FSK. The values are 12 bit and
 * can be derived using formulae in the datasheet.
 */
	void Echo_SetFreqFskNoFRM(unsigned int localOscillator,
						   unsigned int carrier0,
						   unsigned int carrier1) {

		#ifdef ECHO_MODE
  		echoRegisters[ECHO_CONFIG2_REG] |= ECHO_BIT_FRM;

  		echoRegisters[ECHO_F1_REG] = ((localOscillator & 0x0F00)>>8);
  		echoRegisters[ECHO_F0_REG] = (localOscillator & 0x00FF);

  		echoRegisters[ECHO_FT2_REG] = ((carrier0 & 0x0FF0) >> 4);
  		echoRegisters[ECHO_FT1_REG] = ((carrier0 & 0x000F) << 4) | ((carrier1 & 0x0F00)>>8);
  		echoRegisters[ECHO_FT0_REG] = (carrier1 & 0x00FF);
  		
  		echoReg.Ptr		= &echoRegisters[ECHO_CONFIG2_REG];
    #else
      #if ECHO_BANK_A_EN == 1
        if(echoBankStatus & ECHO_ST_BANK_BOTH)
        {
          echoRegisters[ECHO_CONFIG2_REG] |= ECHO_BIT_FRM;

      		echoRegisters[ECHO_F1_REG] = ((localOscillator & 0x0F00)>>8);
      		echoRegisters[ECHO_F0_REG] = (localOscillator & 0x00FF);

      		echoRegisters[ECHO_FT2_REG] = ((carrier0 & 0x0FF0) >> 4);
      		echoRegisters[ECHO_FT1_REG] = ((carrier0 & 0x000F) << 4) | ((carrier1 & 0x0F00)>>8);
      		echoRegisters[ECHO_FT0_REG] = (carrier1 & 0x00FF);
      		
      		echoRegistersB[ECHO_CONFIG2_REG] |= ECHO_BIT_FRM;

      		echoRegistersB[ECHO_F1_REG] = ((localOscillator & 0x0F00)>>8);
      		echoRegistersB[ECHO_F0_REG] = (localOscillator & 0x00FF);

      		echoRegistersB[ECHO_FT2_REG] = ((carrier0 & 0x0FF0) >> 4);
      		echoRegistersB[ECHO_FT1_REG] = ((carrier0 & 0x000F) << 4) | ((carrier1 & 0x0F00)>>8);
      		echoRegistersB[ECHO_FT0_REG] = (carrier1 & 0x00FF);
      		
      	  echoBankStatus |= ECHO_ST_BANK_WR_A | ECHO_ST_BANK_WR_B;
          echoReg.Offset = ECHO_CONFIG2_REG;	
        }
        
        if(echoBankStatus & ECHO_ST_BANK_B)
        {
      		echoRegistersB[ECHO_CONFIG2_REG] |= ECHO_BIT_FRM;

      		echoRegistersB[ECHO_F1_REG] = ((localOscillator & 0x0F00)>>8);
      		echoRegistersB[ECHO_F0_REG] = (localOscillator & 0x00FF);

      		echoRegistersB[ECHO_FT2_REG] = ((carrier0 & 0x0FF0) >> 4);
      		echoRegistersB[ECHO_FT1_REG] = ((carrier0 & 0x000F) << 4) | ((carrier1 & 0x0F00)>>8);
      		echoRegistersB[ECHO_FT0_REG] = (carrier1 & 0x00FF);
      		
      		echoReg.Ptr		= &echoRegistersB[ECHO_CONFIG2_REG]; 
        }else
        {
      		echoRegisters[ECHO_CONFIG2_REG] |= ECHO_BIT_FRM;

      		echoRegisters[ECHO_F1_REG] = ((localOscillator & 0x0F00)>>8);
      		echoRegisters[ECHO_F0_REG] = (localOscillator & 0x00FF);

      		echoRegisters[ECHO_FT2_REG] = ((carrier0 & 0x0FF0) >> 4);
      		echoRegisters[ECHO_FT1_REG] = ((carrier0 & 0x000F) << 4) | ((carrier1 & 0x0F00)>>8);
      		echoRegisters[ECHO_FT0_REG] = (carrier1 & 0x00FF);
      		
      		echoReg.Ptr		= &echoRegisters[ECHO_CONFIG2_REG];
        }        
      #else
    		echoRegistersB[ECHO_CONFIG2_REG] |= ECHO_BIT_FRM;

    		echoRegistersB[ECHO_F1_REG] = ((localOscillator & 0x0F00)>>8);
    		echoRegistersB[ECHO_F0_REG] = (localOscillator & 0x00FF);

    		echoRegistersB[ECHO_FT2_REG] = ((carrier0 & 0x0FF0) >> 4);
    		echoRegistersB[ECHO_FT1_REG] = ((carrier0 & 0x000F) << 4) | ((carrier1 & 0x0F00)>>8);
    		echoRegistersB[ECHO_FT0_REG] = (carrier1 & 0x00FF);
    		
    		echoReg.Ptr		= &echoRegistersB[ECHO_CONFIG2_REG];  
      #endif    
    #endif
		echoCommand		= BUILDCMD(CMD_EIGHT, ECHO_CONFIG2_REG, 1);
		
		echoRegCounter	= 8;

		Echo_StartDelayedConfig();
	}

/****************************************************************************************/


/**---------------------------------------------------------------------------------------
 * Set the input sensitivity on Echo to level. Level must be in the range 0-3. Sets the
 * ILA bits in Echo (see datasheet for more info.)
 */
void Echo_SetRxSensitivity(byte level) {
	#ifdef ECHO_MODE
  	echoRegisters[ECHO_CONFIG3_REG] &= 0b11110011;			/* Clear the ILA bits			*/
  	echoRegisters[ECHO_CONFIG3_REG] |= ((level&0x03)<<2);	/* Copy in 2 bits of level		*/
  		
  	echoReg.Ptr = &echoRegisters[ECHO_CONFIG3_REG];
  #else
    #if ECHO_BANK_A_EN == 1
      if(echoBankStatus & ECHO_ST_BANK_BOTH)
      {
        echoRegisters[ECHO_CONFIG3_REG] &= 0b11110011;			/* Clear the ILA bits			*/
      	echoRegisters[ECHO_CONFIG3_REG] |= ((level&0x03)<<2);	/* Copy in 2 bits of level		*/
      	
      	echoRegistersB[ECHO_CONFIG3_REG] &= 0b11110011;			/* Clear the ILA bits			*/
      	echoRegistersB[ECHO_CONFIG3_REG] |= ((level&0x03)<<2);	/* Copy in 2 bits of level		*/
      		
      	echoBankStatus |= ECHO_ST_BANK_WR_A | ECHO_ST_BANK_WR_B;
        echoReg.Offset = ECHO_CONFIG3_REG;	
      	      	
      }else if(echoBankStatus & ECHO_ST_BANK_B)
      {
      	echoRegistersB[ECHO_CONFIG3_REG] &= 0b11110011;			/* Clear the ILA bits			*/
      	echoRegistersB[ECHO_CONFIG3_REG] |= ((level&0x03)<<2);	/* Copy in 2 bits of level		*/
      		
      	echoReg.Ptr = &echoRegistersB[ECHO_CONFIG3_REG];

      }else
      {
      	echoRegisters[ECHO_CONFIG3_REG] &= 0b11110011;			/* Clear the ILA bits			*/
      	echoRegisters[ECHO_CONFIG3_REG] |= ((level&0x03)<<2);	/* Copy in 2 bits of level		*/
      		
      	echoReg.Ptr = &echoRegisters[ECHO_CONFIG3_REG];
      }        
    #else
    	echoRegistersB[ECHO_CONFIG3_REG] &= 0b11110011;			/* Clear the ILA bits			*/
    	echoRegistersB[ECHO_CONFIG3_REG] |= ((level&0x03)<<2);	/* Copy in 2 bits of level		*/
    		
    	echoReg.Ptr = &echoRegistersB[ECHO_CONFIG3_REG];  
    #endif
  #endif
	


	echoCommand		= BUILDCMD(CMD_ONE, ECHO_CONFIG3_REG, 1);
	echoRegCounter	= 1;
	Echo_StartDelayedConfig();
}

/**---------------------------------------------------------------------------------------
 * Set the output power on Echo to level. Level must be in the range 0-3. Sets the OLA
 * bits in Echo (see datasheet for more info.)
 */
void Echo_SetTxPower(byte level) {
	#ifdef ECHO_MODE
  	echoRegisters[ECHO_CONFIG3_REG] &= 0b11111100;			/* Clear the OLA bits			*/
	  echoRegisters[ECHO_CONFIG3_REG] |= (level&0x03);		/* Copy in 2 bits of level		*/
  		
  	echoReg.Ptr		= &echoRegisters[ECHO_CONFIG3_REG];
  #else
    #if ECHO_BANK_A_EN == 1
      if(echoBankStatus & ECHO_ST_BANK_BOTH)
      {
        echoRegisters[ECHO_CONFIG3_REG] &= 0b11111100;			/* Clear the OLA bits			*/
    	  echoRegisters[ECHO_CONFIG3_REG] |= (level&0x03);		/* Copy in 2 bits of level		*/
    	  
    	  echoRegistersB[ECHO_CONFIG3_REG] &= 0b11111100;			/* Clear the OLA bits			*/
    	  echoRegistersB[ECHO_CONFIG3_REG] |= (level&0x03);		/* Copy in 2 bits of level		*/
        
        echoBankStatus |= ECHO_ST_BANK_WR_A | ECHO_ST_BANK_WR_B;
        echoReg.Offset = ECHO_CONFIG3_REG;
        
      }else if(echoBankStatus & ECHO_ST_BANK_B)
      {
      	echoRegistersB[ECHO_CONFIG3_REG] &= 0b11111100;			/* Clear the OLA bits			*/
    	  echoRegistersB[ECHO_CONFIG3_REG] |= (level&0x03);		/* Copy in 2 bits of level		*/
      		
      	echoReg.Ptr		= &echoRegistersB[ECHO_CONFIG3_REG];
      }else
      {
      	echoRegisters[ECHO_CONFIG3_REG] &= 0b11111100;			/* Clear the OLA bits			*/
    	  echoRegisters[ECHO_CONFIG3_REG] |= (level&0x03);		/* Copy in 2 bits of level		*/
      		
      	echoReg.Ptr		= &echoRegisters[ECHO_CONFIG3_REG];
      }        
    #else
    	echoRegistersB[ECHO_CONFIG3_REG] &= 0b11111100;			/* Clear the OLA bits			*/
  	  echoRegistersB[ECHO_CONFIG3_REG] |= (level&0x03);		/* Copy in 2 bits of level		*/
    		
    	echoReg.Ptr		= &echoRegistersB[ECHO_CONFIG3_REG];  
    #endif
  #endif

	echoCommand		= BUILDCMD(CMD_ONE, ECHO_CONFIG3_REG, 1);	
	echoRegCounter	= 1;

	Echo_StartDelayedConfig();
}


/**---------------------------------------------------------------------------------------
 * Reset the Automatic Gain Control to maximum. Stays at maximum as long as RAGC=1.
 */
void Echo_RagcHigh() {
	#ifdef ECHO_MODE
  	echoRegisters[ECHO_COMMAND_REG] |= ECHO_BIT_RAGC;
  		
  	echoReg.Ptr		= &echoRegisters[ECHO_COMMAND_REG];
  #else
    #if ECHO_BANK_A_EN == 1
      if(echoBankStatus & ECHO_ST_BANK_BOTH)
      {
        echoRegisters[ECHO_COMMAND_REG] |= ECHO_BIT_RAGC;
        echoRegistersB[ECHO_COMMAND_REG] |= ECHO_BIT_RAGC;
        
        echoBankStatus |= ECHO_ST_BANK_WR_A | ECHO_ST_BANK_WR_B;
        echoReg.Offset = ECHO_COMMAND_REG;
      }else if(echoBankStatus & ECHO_ST_BANK_B)
      {
      	echoRegistersB[ECHO_COMMAND_REG] |= ECHO_BIT_RAGC;
      		
      	echoReg.Ptr		= &echoRegistersB[ECHO_COMMAND_REG];

      }else
      {
      	echoRegisters[ECHO_COMMAND_REG] |= ECHO_BIT_RAGC;
      		
      	echoReg.Ptr		= &echoRegisters[ECHO_COMMAND_REG];
      }        
    #else
    	echoRegistersB[ECHO_COMMAND_REG] |= ECHO_BIT_RAGC;
    		
    	echoReg.Ptr		= &echoRegistersB[ECHO_COMMAND_REG]; 
    #endif
  #endif
	
	

	echoCommand		= BUILDCMD(CMD_TWO, ECHO_COMMAND_REG, 1);
	
	echoRegCounter	= 1;

	Echo_StartDelayedConfig();
}

/**---------------------------------------------------------------------------------------
 * Allow Automatic Gain Control to take effect again.
 */
void Echo_RagcLow() {
	#ifdef ECHO_MODE
  	echoRegisters[ECHO_COMMAND_REG] &= ~ECHO_BIT_RAGC;
  		
  	echoReg.Ptr		= &echoRegisters[ECHO_COMMAND_REG];
  #else
    #if ECHO_BANK_A_EN == 1
      if(echoBankStatus & ECHO_ST_BANK_BOTH)
      {
        echoRegisters[ECHO_COMMAND_REG] &= ~ECHO_BIT_RAGC;
        echoRegistersB[ECHO_COMMAND_REG] &= ~ECHO_BIT_RAGC;
        
        echoBankStatus |= ECHO_ST_BANK_WR_A | ECHO_ST_BANK_WR_B;
        echoReg.Offset = ECHO_COMMAND_REG;
      }else if(echoBankStatus & ECHO_ST_BANK_B)
      {
      	echoRegistersB[ECHO_COMMAND_REG] &= ~ECHO_BIT_RAGC;
      		
      	echoReg.Ptr		= &echoRegistersB[ECHO_COMMAND_REG];
      }else
      {
      	echoRegisters[ECHO_COMMAND_REG] &= ~ECHO_BIT_RAGC;
      		
      	echoReg.Ptr		= &echoRegisters[ECHO_COMMAND_REG];
      }        
    #else
    	echoRegistersB[ECHO_COMMAND_REG] &= ~ECHO_BIT_RAGC;
    		
    	echoReg.Ptr		= &echoRegistersB[ECHO_COMMAND_REG]; 
    #endif
  #endif
	

	echoCommand		= BUILDCMD(CMD_TWO, ECHO_COMMAND_REG, 1);
	
	echoRegCounter	= 1;

	Echo_StartDelayedConfig();
}

/**---------------------------------------------------------------------------------------
 * Freeze the Automatic Gain Control to its current value.
 */
void Echo_FagcHigh() {
	#ifdef ECHO_MODE
  	echoRegisters[ECHO_COMMAND_REG] |= ECHO_BIT_FAGC;
  		
  	echoReg.Ptr		= &echoRegisters[ECHO_COMMAND_REG];
  #else
    #if ECHO_BANK_A_EN == 1
      if(echoBankStatus & ECHO_ST_BANK_BOTH)
      {
        echoRegisters[ECHO_COMMAND_REG] |= ECHO_BIT_FAGC;
        echoRegistersB[ECHO_COMMAND_REG] |= ECHO_BIT_FAGC;
        
        echoBankStatus |= ECHO_ST_BANK_WR_A | ECHO_ST_BANK_WR_B;
        echoReg.Offset = ECHO_COMMAND_REG;
        
      }else if(echoBankStatus & ECHO_ST_BANK_B)
      {
      	echoRegistersB[ECHO_COMMAND_REG] |= ECHO_BIT_FAGC;
      		
      	echoReg.Ptr		= &echoRegistersB[ECHO_COMMAND_REG];

      }else
      {
      	echoRegisters[ECHO_COMMAND_REG] |= ECHO_BIT_FAGC;
      		
      	echoReg.Ptr		= &echoRegisters[ECHO_COMMAND_REG];
      }        
    #else
    	echoRegistersB[ECHO_COMMAND_REG] |= ECHO_BIT_FAGC;
    		
    	echoReg.Ptr		= &echoRegistersB[ECHO_COMMAND_REG]; 
    #endif
  #endif	
	

	echoCommand		= BUILDCMD(CMD_ONE, ECHO_COMMAND_REG, 1);
	
	echoRegCounter	= 1;

	Echo_StartDelayedConfig();
}

/**---------------------------------------------------------------------------------------
 * Allow Automatic Gain Control to unfreeze again.
 */
void Echo_FagcLow() {
	#ifdef ECHO_MODE
  	echoRegisters[ECHO_COMMAND_REG] &= ~ECHO_BIT_FAGC;
  		
    echoReg.Ptr		= &echoRegisters[ECHO_COMMAND_REG];
  #else
    #if ECHO_BANK_A_EN == 1
      if(echoBankStatus & ECHO_ST_BANK_BOTH)
      {
        echoRegisters[ECHO_COMMAND_REG] &= ~ECHO_BIT_FAGC;
        echoRegistersB[ECHO_COMMAND_REG] &= ~ECHO_BIT_FAGC;
        
        echoBankStatus |= ECHO_ST_BANK_WR_A | ECHO_ST_BANK_WR_B;
        echoReg.Offset = ECHO_COMMAND_REG;
        
      }else if(echoBankStatus & ECHO_ST_BANK_B)
      {
      	echoRegistersB[ECHO_COMMAND_REG] &= ~ECHO_BIT_FAGC;
      		
        echoReg.Ptr		= &echoRegistersB[ECHO_COMMAND_REG];
      }else
      {
      	echoRegisters[ECHO_COMMAND_REG] &= ~ECHO_BIT_FAGC;
      		
        echoReg.Ptr		= &echoRegisters[ECHO_COMMAND_REG];
      }        
    #else
    	echoRegistersB[ECHO_COMMAND_REG] &= ~ECHO_BIT_FAGC;
    		
      echoReg.Ptr		= &echoRegistersB[ECHO_COMMAND_REG]; 
    #endif
  #endif	
	

	echoCommand		= BUILDCMD(CMD_ONE, ECHO_COMMAND_REG, 1);
	
	echoRegCounter	= 1;

	Echo_StartDelayedConfig();
}


/*-----------------------------------------------------------------------*/
/* Function: Echo_Set_RxOnOff                                                 */
/* Parameters: on - on time 1-15, off - off time 0-7                     */
/* if any parametr is outside of allow scale, then is set the Error bit  */
/* Return: none                                                          */
/*                                                                       */
/* Function Sets actual RXONOFF registr                                  */
/*-----------------------------------------------------------------------*/
void Echo_Set_RxOnOff(byte on, byte off)
{
  byte temp = 0;
  if((on < 1) || (on > 15) || (off > 7))
  {
    status.Bits.Error = 1;
  }else
  {
  #ifdef ECHO_MODE
    echoRegisters[ECHO_RXONOFF_REG] = ((on & 0x0f)<<3) | (off & 0x07) | ECHO_BIT_BANK;   // only for test
    echoReg.Ptr		= &echoRegisters[ECHO_RXONOFF_REG];
  #else
    #if ECHO_BANK_A_EN == 1
      if(echoBankStatus & ECHO_ST_BANK_BOTH)
      {
        echoRegisters[ECHO_RXONOFF_REG] = ((on & 0x0f)<<3) | (off & 0x07) | ECHO_BIT_BANK;
        echoRegistersB[ECHO_RXONOFF_REG] = ((on & 0x0f)<<3) | (off & 0x07) | ECHO_BIT_BANK;
        echoReg.Offset = ECHO_RXONOFF_REG;
        echoBankStatus |= (ECHO_ST_BANK_WR_A | ECHO_ST_BANK_WR_B);
      }else if(echoBankStatus & ECHO_ST_BANK_B)
      {
        echoRegistersB[ECHO_RXONOFF_REG] = ((on & 0x0f)<<3) | (off & 0x07) | ECHO_BIT_BANK;
        echoReg.Ptr		= &echoRegistersB[ECHO_RXONOFF_REG];
      }else
      {
        echoRegisters[ECHO_RXONOFF_REG] = ((on & 0x0f)<<3) | (off & 0x07) | ECHO_BIT_BANK;
        echoReg.Ptr		= &echoRegisters[ECHO_RXONOFF_REG];
      }  
    #else
      echoRegistersB[ECHO_RXONOFF_REG] = ((on & 0x0f)<<3) | (off & 0x07) | ECHO_BIT_BANK;
      echoReg.Ptr		= &echoRegistersB[ECHO_RXONOFF_REG];
    #endif
  #endif
  
  echoCommand		= BUILDCMD(CMD_ONE, ECHO_RXONOFF_REG, 1);
  echoRegCounter	= 1;

	Echo_StartDelayedConfig();
  }
}

/**---------------------------------------------------------------------------------------
 * Enable the RSSI module and automatically read RSSI during reception of messages.
 * The received RSSI reading will be stored in the buffer along with the message.
 */
void Echo_EnableRSSI(void) {
	status.Bits.RSSI_Enabled = 1;

	#ifdef ECHO_MODE
    echoRegisters[ECHO_COMMAND_REG] |= ECHO_BIT_RSSIE;
  		
    echoReg.Ptr	= &echoRegisters[ECHO_COMMAND_REG];
  #else
    #if ECHO_BANK_A_EN == 1
        echoRegistersB[ECHO_COMMAND_REG] |= ECHO_BIT_RSSIE;
        echoRegisters[ECHO_COMMAND_REG] |= ECHO_BIT_RSSIE;		
        echoReg.Offset = ECHO_COMMAND_REG;
        echoBankStatus |= (ECHO_ST_BANK_WR_A | ECHO_ST_BANK_WR_B);      
    #else
      echoRegistersB[ECHO_COMMAND_REG] |= ECHO_BIT_RSSIE;
    		
      echoReg.Ptr	= &echoRegistersB[ECHO_COMMAND_REG]; 
    #endif
  #endif	
	

	echoCommand	= BUILDCMD(CMD_ONE, ECHO_COMMAND_REG, 1);
	
	echoRegCounter = 1;

	Echo_StartDelayedConfig();
}

/**---------------------------------------------------------------------------------------
 * Disable the RSSI module and stop automatically reading the RSSI during reception.
 */
void Echo_DisableRSSI(void) {
	status.Bits.RSSI_Enabled = 0;

	#ifdef ECHO_MODE
    echoRegisters[ECHO_COMMAND_REG] &= ~ECHO_BIT_RSSIE;
  		
    echoReg.Ptr		= &echoRegisters[ECHO_COMMAND_REG];
  #else
    #if ECHO_BANK_A_EN == 1
        echoRegistersB[ECHO_COMMAND_REG] &= ~ECHO_BIT_RSSIE;
        echoRegisters[ECHO_COMMAND_REG] &= ~ECHO_BIT_RSSIE;	
        echoReg.Offset = (word) ECHO_COMMAND_REG;
        echoBankStatus |= (ECHO_ST_BANK_WR_A | ECHO_ST_BANK_WR_B);        
    #else
      echoRegistersB[ECHO_COMMAND_REG] &= ~ECHO_BIT_RSSIE;
    		
      echoReg.Ptr		= &echoRegistersB[ECHO_COMMAND_REG]; 
    #endif
  #endif
  

  echoCommand		= BUILDCMD(CMD_ONE, ECHO_COMMAND_REG, 1);
	echoRegCounter	= 1;

	Echo_StartDelayedConfig();
}



/**---------------------------------------------------------------------------------------
 * Change Echo's registers directly. 'numReg' must be either 1,2,4 or 8. Otherwise, status
 * bit Error is set. 'startReg' is the Echo register number to start at. 'regData' the
 * buffer to read to/write from. 'readWrite' is 0 to read, 1 to write.
 *
 * Registers are written sequentially through starting at 'startReg' for 'numReg'
 * locations, as described in the Echo datasheet.
 */
void Echo_ChangeConfig(byte numReg,
					   byte startReg,
					   byte *regData,
					   byte readWrite
					   ) {
	byte i;
	byte n;
	if (startReg>15 || readWrite>1) {
		status.Bits.Error = 1;
		return;
	}
	switch (numReg) {
		case 1:
			echoRegCounter = 1;
			n=0;
			break;
		case 2:
			echoRegCounter = 2;
			n=1;
			break;
		case 4:
			echoRegCounter = 4;
			n=2;
			break;
		case 8:
			echoRegCounter = 8;
			n=3;
			break;
		default:
			status.Bits.Error = 1;
			return;
	}

	echoCommand = BUILDCMD(n,startReg,readWrite);
	echoReg.Ptr = regData;
	
	/* Copy regData into local copy of Echo's registers									*/
	if(readWrite == 1)
	{
  	for(i=0; i<numReg; i++) {
    	#ifdef ECHO_MODE
     	  echoRegisters[startReg+i] = regData[i];
      #else
        #if ECHO_BANK_A_EN == 1
          if(echoBankStatus & ECHO_ST_BANK_B)
            echoRegistersB[startReg+i] = regData[i];
          else
            echoRegisters[startReg+i] = regData[i];
        #else
          echoRegistersB[startReg+i] = regData[i]; 
        #endif
      #endif
  	}
  }	
  Echo_StartDelayedConfig();
}


/**---------------------------------------------------------------------------------------
 * Change Echo's registers bank to BANK A or BANK B or both BANKs.
 * If is set non exist Bank then bit Error is set. This function must be used before
 * each transmit if is used to both bank for receiveing messages.
 * Parameter can be have this three values -  ECHO_BANK_A
 *                                            ECHO_BANK_B
 *                                            ECHO_BANK_BOTH
 */
void Echo_ChangeBank(byte bank)
{
  #ifdef ECHO_MODE                     // Driver work in simple mode - bank must be set always to A
    if(bank != ECHO_BANK_A)
    status.Bits.Error = 1;             // If you sets other than bank A then set Error
  #else
    #if ECHO_BANK_B_EN == 1 && ECHO_BANK_A_EN == 0    // Driver work in second simple mode - bank must be set always to B
      if(bank != ECHO_BANK_B) status.Bits.Error = 1; // If you sets other than bank B then set Error
      p_echoBank = (tSWITCH_BANK_INFO*) &echoBankBConst;
    #endif
    
    #if ECHO_BANK_B_EN == 1 && ECHO_BANK_A_EN == 1
    switch(bank){
    case ECHO_BANK_A:    /* Bank A is required to set */
        if(echoBankStatus & ECHO_ST_BANK_BOTH)
        {
          // now are both enabled
          echoRegistersB[ECHO_RXONOFF_REG] &= ~ECHO_BIT_BANK;  /* switch off bank B in shadow reg*/
          echoBankStatus &= ~ECHO_ST_BANK_BOTH;                /* clear status bit of both mode */ 
          echoBankStatus |=  ECHO_ST_BANK_SWITCH | ECHO_ST_BANK_WR_B | ECHO_ST_BANK_A; /* set status bit of write to bank B, switch banks and Bank A is active */
          status.Bits.SwitchMode = 0;
        }else if(echoBankStatus & ECHO_ST_BANK_B)
        {
          // now is bank B selected
          echoRegistersB[ECHO_RXONOFF_REG] &= ~ECHO_BIT_BANK; /* switch off bank B in shadow reg*/
          echoRegisters[ECHO_RXONOFF_REG] |= ECHO_BIT_BANK;   /* switch on bank A in shadow reg*/
          echoBankStatus &= ~ECHO_ST_BANK_B;                  /* clear status bit of bank B mode */
          echoBankStatus |=  ECHO_ST_BANK_A | ECHO_ST_BANK_SWITCH | ECHO_ST_BANK_WR_B | ECHO_ST_BANK_WR_A; /* set status bit of write to bank B and A , switch banks and Bank A is active */
        } else echoBankStatus &= ~(ECHO_ST_BANK_SWITCH | ECHO_ST_BANK_WR_B | ECHO_ST_BANK_WR_A);  /* Do nothing - bank A is already sets */
        
        p_echoBank = (tSWITCH_BANK_INFO*) &echoBankAConst;  /* set up pointer to Bank A values */
        break;
        
    case ECHO_BANK_B:     /* Bank A is required to set */
        if(echoBankStatus & ECHO_ST_BANK_BOTH)
        {
          // now are both enabled
          echoRegisters[ECHO_RXONOFF_REG] &= ~ECHO_BIT_BANK;
          echoBankStatus &= ~ECHO_ST_BANK_BOTH;
          echoBankStatus |=  ECHO_ST_BANK_SWITCH | ECHO_ST_BANK_WR_A | ECHO_ST_BANK_B;
          status.Bits.SwitchMode = 0;
        }else if(echoBankStatus & ECHO_ST_BANK_A)
        {
          // now is bank A selected
          echoRegisters[ECHO_RXONOFF_REG] &= ~ECHO_BIT_BANK;
          echoRegistersB[ECHO_RXONOFF_REG] |= ECHO_BIT_BANK;
          echoBankStatus &= ~ECHO_ST_BANK_A;
          echoBankStatus |=  ECHO_ST_BANK_B | ECHO_ST_BANK_SWITCH | ECHO_ST_BANK_WR_A | ECHO_ST_BANK_WR_B;
        } else echoBankStatus &= ~(ECHO_ST_BANK_SWITCH | ECHO_ST_BANK_WR_B | ECHO_ST_BANK_WR_A);
        // now is bank B selected
        p_echoBank = (tSWITCH_BANK_INFO*) &echoBankBConst;
        
        break;
    case ECHO_BANK_BOTH:
        if(echoBankStatus & ECHO_ST_BANK_BOTH)
        {
          // now are both enabled
          echoBankStatus &= ~(ECHO_ST_BANK_SWITCH | ECHO_ST_BANK_WR_B | ECHO_ST_BANK_WR_A);
        }else if(echoBankStatus & ECHO_ST_BANK_A)
        {
          // now is bank A selected
          echoRegistersB[ECHO_RXONOFF_REG] |= ECHO_BIT_BANK;
          echoBankStatus &= ~ECHO_ST_BANK_A;
          echoBankStatus |=  ECHO_ST_BANK_SWITCH | ECHO_ST_BANK_WR_B | ECHO_ST_BANK_BOTH;
        }else
        {
          // now is bank B selected
          echoRegisters[ECHO_RXONOFF_REG] |= ECHO_BIT_BANK;
          echoBankStatus &= ~ECHO_ST_BANK_B;
          echoBankStatus |=  ECHO_ST_BANK_SWITCH | ECHO_ST_BANK_WR_A | ECHO_ST_BANK_BOTH;
        }
        p_echoBank = (tSWITCH_BANK_INFO*) &echoBankAConst;    
        status.Bits.SwitchMode = 1;
        break;
    default:
        status.Bits.Error = 1;    /* If is set other bank than ECHO have then error flag is set */
        break;
    }
    
    if(echoBankStatus & ECHO_ST_BANK_SWITCH)  /* Switch bank ? */
    {
      //ECHO_TPMxMODH = _ECHO_MODH;					/* Timer modulus = a bit-time			*/
  	  //ECHO_TPMxMODL = _ECHO_MODL;
  	
  	  //ECHO_TPMxCxVH = _ECHO_MODH;					/* Compare = full bit-time				*/
  	  //ECHO_TPMxCxVL = _ECHO_MODL;
     
      Echo_StartDelayedConfig();          /* Start switching banks */

    }
    #endif
  #endif
}
